@@ -3,14 +3,14 @@
  *
  * Author:  Kazu Yamamoto <Kazu@Mew.org>
  * Created: Dec  8, 1994
- * Revised: Jan 17, 2002
+ * Revised: Jul 10, 2001
  *
  * Code:
  */
 
 #include "mew.h"
 
-private char version_message[] = "version 2.3 20021017 Kazu Yamamoto";
+private char version_message[] = "version 3.1 20021120 Kazu Yamamoto";
 
 #ifdef HAVE_UNISTD_H
 #include <sys/types.h>
@@ -58,14 +58,18 @@ private void help(const char *);
 private void version(const char *);
 private int  read_text(byte *, unsigned int);
 private int  read_binary(byte *, unsigned int);
+#ifdef HAVE_LIBZ
 private int  deflate_gzip(byte *, unsigned int);
+#endif /* HAVE_LIBZ */
 private int  encode_base64(byte *, unsigned int);
 private void write_base64(void);
 private void base64_encode(int, int);
 private void gzip64_encode(int, int);
 private int  read_base64(byte *, unsigned int);
 private int  decode_base64(byte *, unsigned int);
+#ifdef HAVE_LIBZ
 private int  inflate_gzip(byte *, unsigned int);
+#endif /* HAVE_LIBZ */
 private void write_text(void);
 private void write_binary(void);
 private void base64_decode(int);
@@ -148,7 +152,9 @@ struct {
 	unsigned int e_length;
 } E_ctx;
 
+#if HAVE_LIBZ
 private byte e_buf1[BUFSIZ];
+#endif  /* HAVE_LIBZ */
 private byte e_buf2[BUFSIZ];
 private byte e_buf3[BUFSIZ];
 
@@ -448,7 +454,9 @@ struct {
 } D_ctx;
 
 private byte d_buf1[BUFSIZ];
+#if HAVE_LIBZ
 private byte d_buf2[BUFSIZ];
+#endif /* HAVE_LIBZ */
 private byte d_buf3[BUFSIZ];
 
 private signed char
