@@ -3,14 +3,14 @@
  *
  * Author:  Kazu Yamamoto <Kazu@Mew.org>
  * Created: Jul  7, 2000
- * Revised: Jan 17, 2002
+ * Revised: Oct 31, 2001
  *
  * Code:
  */
 
 #include "mew.h"
 
-private char version_message[] = "version 2.3 20021017 Kazu Yamamoto";
+private char version_message[] = "version 3.1 20021120 Kazu Yamamoto";
 
 #ifdef HAVE_UNISTD_H
 # include <sys/types.h>
@@ -59,7 +59,6 @@ private char version_message[] = "version 2.3 20021017 Kazu Yamamoto";
  */
 
 #define MAILHOME "Mail"
-#define NEWSHOME "News"
 #define CHDIR    "CD:"
 #define INBOX    "+inbox"
 #define STDINFILE "-"
@@ -67,7 +66,7 @@ private char version_message[] = "version 2.3 20021017 Kazu Yamamoto";
 
 /* 16k, my experience says that 8k is not enough. */
 #define MAX_HEADER         16384
-#define MAX_FIELD_LEN          3
+#define MAX_FIELD_LEN          5
 #define MAX_FOLDER            16
 #define NUMBER_OF_FILE       128
 #define MAX_ARGC	      16
@@ -93,7 +92,6 @@ struct fld_rng {
 private char Buf_header[MAX_HEADER];
 private char Buf_filenm[MAX_FILE_NAME_LEN];
 private char *Mail_home;
-private char *News_home;
 private char Current_folder[MAX_FILE_NAME_LEN];
 private struct fld_rng Folders[MAX_FOLDER];
 private char *Argvec[MAX_ARGC];
@@ -105,6 +103,7 @@ private char *Input_file = NULL;
 
 private int All_fields = 0;
 private int Field_len = MAX_FIELD_LEN;
+private int PrintNumOfMsg = NO;
 
 /****************************************************************
  *
@@ -154,7 +153,6 @@ private const char *
 help_message[] = {
 	" -a            Print all necessary fields when picking.",
 	" -b  <mdir>    Mail home.",
-	" -c  <ndir>    News home.",
 	" -d  <field>   A field to be extracted.",
 	" -e  <command> An external command to fetch mailbox.",
 	" -m  <options> The second argument for the external command.",
@@ -173,8 +171,8 @@ help_message[] = {
 	"          | '<pattern>&<pattern>' ",
 	"          | '<pattern>|<pattern>' ",
 	"",
-	"Default is mdir = ~/Mail, ndir = ~/News",
-	"length = 3, folder = +inbox, range = 0",
+	"Default is mdir = ~/Mail,",
+	"length = 5, folder = +inbox, range = 0",
 	"field = To:,From:,Subject:,Date:,Content-Type:,Message-ID:,X-Mew-UIDL:,In-Reply-To:,References:",
 	NULL
 };
@@ -261,13 +259,6 @@ ch_folder(char *folder)
 		if (chdir(p) != 0)
 			warn_exit("can't change folder to %s.\n", folder);
 		break;
-	case '=':
-		ch_mail_home(News_home);
-		if (p == NUL)
-			break;
-		if (chdir(p) != 0)
-			warn_exit("can't change folder to %s.\n", folder);
-		break;
 	case '~':
 		ch_home();
 		if (*p == NUL)
@@ -635,7 +626,7 @@ private unsigned int *Scan_ctx_buf;
 private int
 scan_numfile(char *s, unsigned int first, unsigned int last) {
 	unsigned int num = 0;
-	
+
 	if (*s == NUL)
 		return 0;
 
@@ -697,7 +688,11 @@ scan_folder(struct fld_rng *fr) {
 			Scan_ctx_buf[i] = Scan_ctx_buf[num - fr->num + i] ;
 		num = fr->num;
 	}
-	
+
+	if (PrintNumOfMsg == YES && num > 0 && Scan_ctx_folders == 1) {
+		printf("NumOfMsg: %d\n", num);
+		fflush(stdout);
+	}
 	return num;
 }
 
@@ -728,10 +723,19 @@ scan_getfile(char **filename, char **foldername)
 	*foldername = Folders[j - 1].folder;
 					    
 	while (i < Scan_ctx_messages) {
-		sprintf(Buf_filenm, "%d", *(Scan_ctx_buf + i++));
+		sprintf(Buf_filenm, "%d", *(Scan_ctx_buf + i));
 		fp = fopen(Buf_filenm, "r");
-		if (fp == NULL) continue;
+		if (fp == NULL) {
+			/* for invalid cache messages */
+			sprintf(Buf_filenm, "0%d", *(Scan_ctx_buf + i));
+			fp = fopen(Buf_filenm, "r");
+			if (fp == NULL) {
+			  i++;
+			  continue;
+			}
+		}
 		*filename = Buf_filenm;
+		i++;
 		return fp;
 	}
 	goto again;
@@ -913,7 +917,6 @@ main(int argc, char **argv)
 
 	warn_prog = progname;
 	STRDUP(Mail_home, MAILHOME);
-	STRDUP(News_home, NEWSHOME);
 
 	while ((optc = Getopt(argc, argv, "ab:c:d:e:f:hi:l:m:np:s:vw:")) != EOF) {
 		switch (optc) {
@@ -924,7 +927,7 @@ main(int argc, char **argv)
 			STRDUP(Mail_home, Optarg);
 			break;
 		case 'c':
-			STRDUP(News_home, Optarg);
+			/* just ignore for backward compatibility */
 			break;
 		case 'd':
 			STRDUP(Key_field, Optarg);
@@ -935,6 +938,7 @@ main(int argc, char **argv)
 			break;
 		case 'f':
 			STRDUP(fields, Optarg);
+			PrintNumOfMsg = YES;
 			break;
 		case 'h':
 			help(progname);
@@ -1011,6 +1015,7 @@ main(int argc, char **argv)
 	if (pattern != NULL) {
 		pattern_init(pattern);
 		func_print = print_for_pick;
+		PrintNumOfMsg = NO;
 	} else if (Key_field != NULL)
 		func_print = print_for_sort;
 	else
