@@ -16,6 +16,10 @@
 #include <ctype.h>
 #include <stdarg.h>
 
+#if !HAVE_MEMMOVE
+#define memmove(d, s, n) bcopy((s), (d), (n))
+#endif
+
 char *warn_prog;
 
 public char *
