@@ -8,9 +8,12 @@
 ;; (eval-after-load "mew" '(require 'mew-nmz))
 ;;
 
-(defconst mew-nmz-version "mew-nmz.el 0.61")
+(defconst mew-nmz-version "mew-nmz.el 0.70")
+
+(eval-when-compile
+  (require 'mew)
+  (defvar namazu-history))
 
-(eval-when-compile (require 'mew))
 (and (locate-library "namazu")
      (eval-when-compile (require 'namazu)))
 
@@ -80,6 +83,21 @@ A good example is \"+from:\".")
   "*Use namazu-mode from mew.")
 (defvar mew-nmz-namazu-full-window t "*Use namazu-mode full window.")
 
+;; internal variable, don't modify.
+(defconst mew-nmz-namazu-version1-str "^  Search Program of Namazu v1\.[34]\.")
+(defconst mew-nmz-namazu-version2-str "^namazu of Namazu [1-9.]+")
+(defvar mew-nmz-pick-pattern-hist nil)
+(defvar mew-nmz-gather-header-list nil)
+(defvar mew-nmz-namazu-last-folder nil)
+(defvar mew-nmz-indexed-folders nil)
+
+(defvar mew-nmz-mknmz-process nil)
+(defvar mew-nmz-mknmz-process-file nil)
+(defvar mew-nmz-mknmz-process-folder nil)
+(make-variable-buffer-local 'mew-nmz-mknmz-process)
+(make-variable-buffer-local 'mew-nmz-mknmz-process-file)
+(make-variable-buffer-local 'mew-nmz-mknmz-process-folder)
+
 ;; Key Bind
 (add-hook 'mew-summary-mode-hook
 	  (lambda ()
@@ -121,36 +139,15 @@ If executed with 'C-u', jump to original message folder and number."
 	(message "Original message at %s/%s" fld msg)
 	(when arg
 	  (mew-nmz-goto-folder-msg fld msg)
-	  (message "Original message at %s/%s... jump done." fld msg))))))
-
-;; internal variable, don't modify.
-(defconst mew-nmz-namazu-version1-str "^  Search Program of Namazu v1\.[34]\.")
-(defconst mew-nmz-namazu-version2-str "^namazu of Namazu [1-9.]+")
-(defvar mew-nmz-pick-pattern-hist nil)
-(defvar mew-nmz-gather-header-list nil)
-(defvar mew-nmz-namazu-last-folder nil)
-(defvar mew-nmz-indexed-folders nil)
-
-(defvar mew-nmz-mknmz-process nil)
-(defvar mew-nmz-mknmz-process-file nil)
-(defvar mew-nmz-mknmz-process-folder nil)
-(make-variable-buffer-local 'mew-nmz-mknmz-process)
-(make-variable-buffer-local 'mew-nmz-mknmz-process-file)
-(make-variable-buffer-local 'mew-nmz-mknmz-process-folder)
+	  (message "Original message at %s/%s...jump done" fld msg))))))
 
 ;; macros
 (defmacro mew-nmz-expand-folder (fld)
   "Convert fld to namazu-index-dir."
-  `(cond
-    ((mew-folder-mailp ,fld)
+  `(when (mew-folder-localp ,fld)
      (expand-file-name
       (substring ,fld 1 nil)
-      (expand-file-name mew-nmz-index-mail mew-nmz-index-path)))
-    ((mew-folder-local-newsp ,fld)
-     (expand-file-name
-      (substring ,fld 1 nil)
-      (expand-file-name mew-nmz-index-news mew-nmz-index-path)))
-    (t nil)))
+      (expand-file-name mew-nmz-index-mail mew-nmz-index-path))))
 
 ;;
 ;; Namazu Version check.
@@ -189,14 +186,14 @@ If executed with 'C-u', remove index files at the specified folder."
       (if (not (mew-which mew-nmz-prog-mknmz exec-path))
 	  (message "Please install mknmz.")
 	(and current-prefix-arg (setq remove t))
-	(if  (not fld)
-	    (setq fld (mew-input-folder (mew-summary-folder-name 'ext)))
+	(if (not fld)
+	    (setq fld (mew-input-folder (mew-sinfo-get-case)
+					(mew-summary-folder-name 'ext)))
 	  (setq fld (directory-file-name fld)))
 	(setq figfld (mew-nmz-have-figure-folder fld))
-	(if (or (mew-folder-newsp fld)
+	(if (or (mew-folder-remotep fld)
 		(mew-folder-virtualp fld)
-		(mew-nmz-skip-folder fld)
-		(and mew-nmz-mknmz-skip-news (mew-folder-local-newsp fld)))
+		(mew-nmz-skip-folder fld))
 	    (and msgenb (message "Can't make namazu index in %s." fld))
 	  (if (and (not nocheck) (not remove) (mew-nmz-index-newp fld))
 	      (message "%s has a newer namazu index." fld)
@@ -263,7 +260,7 @@ If executed with 'C-u', remove index files at the specified folder."
 			(if callp
 			    (progn
 			      (mew-nmz-timestamp-new fld)
-			      (message "Mew mknmz (%s) ..." fld)
+			      (message "Mew mknmz (%s)..." fld)
 			      (mew-piolet
 			       mew-cs-autoconv mew-cs-text-for-write
 			       (apply (function call-process)
@@ -276,7 +273,7 @@ If executed with 'C-u', remove index files at the specified folder."
 					   (member fld mew-nmz-use-gcnmz-folders))
 				       (mew-which mew-nmz-prog-gcnmz exec-path))
 				  (progn
-				    (message "Mew mknmz (%s) ... refresh" fld)
+				    (message "Mew mknmz (%s)...refresh" fld)
 				    (mew-piolet
 				     mew-cs-autoconv mew-cs-text-for-write
 				     (apply (function call-process)
@@ -297,12 +294,12 @@ If executed with 'C-u', remove index files at the specified folder."
 			      (delete-file tmpf)
 			      (goto-char (point-min))
 			      (if (search-forward-regexp "^ERROR:.*$" nil t)
-				  (message "Mew mknmz (%s) ... %s." fld (mew-match 0))
-				(message "Mew mknmz (%s) ... done." fld))
+				  (message "Mew mknmz (%s)...%s" fld (match-string 0))
+				(message "Mew mknmz (%s) ...done" fld))
 			      (set-buffer-modified-p nil)
 			      (kill-buffer (current-buffer)))
 			  ;; start-process
-			  (and msgenb (message "Mew mknmz (%s) ..." fld))
+			  (and msgenb (message "Mew mknmz (%s)..." fld))
 			  (mew-nmz-timestamp-new fld)
 			  (mew-piolet
 			   mew-cs-autoconv mew-cs-text-for-write
@@ -329,15 +326,15 @@ If executed with 'C-u', remove index files at the specified folder."
 	  msg)
       (goto-char (point-min))
       (if (search-forward-regexp "^ERROR:.*$" nil t)
-	  (setq msg (format "Mew mknmz (%s) ... %s." fld (mew-match 0)))
-	(setq msg (format "Mew mknmz (%s) ... done" fld))
+	  (setq msg (format "Mew mknmz (%s)...%s" fld (match-string 0)))
+	(setq msg (format "Mew mknmz (%s)...done" fld))
 	(if (not (or (null mew-nmz-indexed-folders)
 		     (member fld mew-nmz-indexed-folders)))
 	    (setq mew-nmz-indexed-folders
 		  (cons fld mew-nmz-indexed-folders))))
       (and (file-readable-p tmpfile)
 	   (file-writable-p tmpfile)
-	   (condition-case err
+	   (condition-case nil
 	       (progn
 		 (delete-file tmpfile)
 		 (mew-nmz-timestamp-rename fld))
@@ -370,14 +367,14 @@ If executed with 'C-u', remove index files at the specified folder."
   (interactive "P")
   (if (not arg)
       ()
-    (message "folder setup ...")
-    (mew-folder-setup (interactive-p))
-    (message "folder setup ... Done."))
+    (message "folder setup...")
+    (mew-local-update (interactive-p))
+    (message "folder setup...done"))
   (save-excursion
-    (let ((flist mew-folder-list))
+    (let ((flist mew-local-folder-alist))
       (while (car flist)
-	(condition-case err
-	    (mew-nmz-mknmz (car flist) nil 'call)
+	(condition-case nil
+	    (mew-nmz-mknmz (car (car flist)) nil 'call)
 	  (error nil))
 	(setq flist (cdr flist)))))
   (message "Namazu make index done."))
@@ -390,7 +387,7 @@ If executed with 'C-u', remove index files at the specified folder."
        (save-excursion
 	 (if (and (mew-summary-mark-collect
 		   mew-nmz-mark-unindexed (point-min) (point-max))
-		  (y-or-n-p (format "Unmark '%c' ? " mew-nmz-mark-unindexed)))
+		  (y-or-n-p (format "Unmark '%c'? " mew-nmz-mark-unindexed)))
 	     (mew-mark-undo-mark mew-nmz-mark-unindexed 'nomsg))
 	 (let* ((ufname
 		 (expand-file-name
@@ -406,13 +403,13 @@ If executed with 'C-u', remove index files at the specified folder."
 	       (message "checking %s..." (file-name-nondirectory ufname))
 	       (insert-file-contents ufname)
 	       (while (re-search-forward "/\\([1-9][0-9]*\\)$" nil t)
-		 (setq msgnums (cons (string-to-number (mew-match 1)) msgnums))))
-	     (message "checking %s ..." (buffer-name))
+		 (setq msgnums (cons (string-to-number (match-string 1)) msgnums))))
+	     (message "checking %s..." (buffer-name))
 	     (goto-char (point-min))
 	     (while (not (eobp))
 	       (if (and
 		    (looking-at "^ *\\([1-9][0-9]*\\)")
-		    (not (memq (string-to-number (mew-match 1)) msgnums))
+		    (not (memq (string-to-number (match-string 1)) msgnums))
 		    (not (mew-in-decode-syntax-p)))
 		   (progn
 		     (setq umsgs (1+ umsgs))
@@ -442,17 +439,22 @@ If executed with 'C-u', search *child* message."
 		   (list mew-message-id:)))
 	(message (if child "children" "parent"))
 	(refilefld (copy-sequence mew-nmz-search-parent-folder))
+	(proto (or (mew-proto-to-refile (mew-summary-folder-name 'ext))
+		   "+"))
+	(case (mew-sinfo-get-case))
 	refiledir mess ref pid rh)
     (if mid
 	(setq pid (list mid) idh nil)
       (if (not (or (mew-summary-message-number) (mew-syntax-number)))
-	  (error "No message here.")
+	  (message "No message here.")
 	(save-excursion
 	  (mew-summary-display nil)
 	  (setq mess (or (mew-cache-hit
 			  (mew-summary-folder-name) (mew-summary-message-number))
 			 (mew-buffer-message)))
 	  (set-buffer mess)
+	  (mew-sinfo-set-case case)
+	  (mew-sinfo-set-proto proto)
 	  (setq refilefld (append (car (mew-refile-guess nil t)) refilefld))
 	  (if child
 	      (setq idh (car (cdr idh)))
@@ -461,7 +463,7 @@ If executed with 'C-u', search *child* message."
 	    (setq rh (car idh))
 	    (setq ref (mew-header-get-value rh))
 	    (while (and ref (string-match "<\\([^>]+\\)>" ref))
-	      (setq pid (cons (mew-match 1 ref) pid))
+	      (setq pid (cons (concat "\"" (match-string 1 ref) "\"") pid))
 	      (setq refilefld
 		    (cons (nth 1 (assoc (car pid) mew-refile-msgid-alist)) refilefld))
 	      (setq ref (substring ref (match-end 0))))
@@ -472,82 +474,82 @@ If executed with 'C-u', search *child* message."
 		(delete nil (mapcar
 			     (lambda (x)
 			       (mew-nmz-expand-folder x))
-			     refilefld)))
-	  (if (null pid)
-	      (error "No required header.")))))
-    (if (mew-syntax-number)
-	(while (not (mew-summary-message-number))
-	  (forward-line -1)))
-    (set-marker mew-summary-inbox-position (point) (current-buffer))
-    (let ((pattern1 "")
-	  (pattern2 "")
-	  (addpattern (if child "+in-reply-to:" "+message-id:"))
-	  (range nil))
-      (if (not child)
+			     refilefld))))))
+    (if (null pid)
+	(message "No required header.")
+      (if (mew-syntax-number)
+	  (while (not (mew-summary-message-number))
+	    (forward-line -1)))
+      (set-marker mew-summary-inbox-position (point) (current-buffer))
+      (let ((pattern1 "")
+	    (pattern2 "")
+	    (addpattern (if child "+in-reply-to:" "+message-id:"))
+	    (range nil))
+	(if (not child)
+	    (setq pattern1 (concat addpattern (car pid)))
 	  (setq pattern1 (concat addpattern (car pid)))
-	(setq pattern1 (concat addpattern (car pid)))
-	(setq addpattern "+references:")
-	(setq pattern1 (concat pattern1 " | " addpattern (car pid))))
-      (setq pid (delete (car pid) pid))
-      (while pid
-	(if (> (length (concat pattern2 addpattern (car pid)))
-	       mew-nmz-query-max-length)
-	    (setq pid nil)
-	  (setq pattern2 (concat pattern2 addpattern (car pid)))
-	  (setq addpattern (if child " | +references:" " | +message-id:"))
-	  (setq pid (delete (car pid) pid))))
-      (message "Searching %s ..." message)
-      (let ((pattern (list pattern1 pattern2)))
-	(while (and (null range) pattern)
-	  (if mid
-	      ()
-	    (message "Searching %s ... %s" message (mew-join ", " refilefld))
-	    (setq range (mew-nmz-multi-pick refiledir (car pattern)))
+	  (setq addpattern "+references:")
+	  (setq pattern1 (concat pattern1 " | " addpattern (car pid))))
+	(setq pid (delete (car pid) pid))
+	(while pid
+	  (if (> (length (concat pattern2 addpattern (car pid)))
+		 mew-nmz-query-max-length)
+	      (setq pid nil)
+	    (setq pattern2 (concat pattern2 addpattern (car pid)))
+	    (setq addpattern (if child " | +references:" " | +message-id:"))
+	    (setq pid (delete (car pid) pid))))
+	(message "Searching %s..." message)
+	(let ((pattern (list pattern1 pattern2)))
+	  (while (and (null range) pattern)
+	    (if mid
+		()
+	      (message "Searching %s...%s" message (mew-join ", " refilefld))
+	      (setq range (mew-nmz-multi-pick refiledir (car pattern)))
+	      (if range
+		  (catch 'detect
+		    (while refilefld
+		      (if (null (setq idh (assoc (car refilefld) range)))
+			  ()
+			(setq fld (car idh))
+			(if child
+			    (setq range (cdr idh))
+			  (setq range (nreverse (cdr idh))))
+			(throw 'detect t))
+		      (setq refilefld (cdr refilefld)))
+		    nil)))
 	    (if range
-		(catch 'detect
-		  (while refilefld
-		    (if (null (setq idh (assoc (car refilefld) range)))
-			()
-		      (setq fld (car idh))
-		      (if child
-			  (setq range (cdr idh))
-			(setq range (nreverse (cdr idh))))
-		      (throw 'detect t))
-		    (setq refilefld (cdr refilefld)))
-		  nil)))
-	  (if range
-	      ()
-	    ;; all folder search
-	    (message "Searching %s ... all folders" message)
-	    (setq range (mew-nmz-multi-pick
-			 (mew-nmz-expand-folder-regexp "*")
-			 (car pattern) t))
-	    (if (null range)
-		(setq pattern (cdr pattern))
-	      (setq fld (car (car range)))
-	      (setq range (cdr (car range)))
-	      (if (not child) (setq range (nreverse range)))
-	      ))))
-      (if (null range)
-	  (error "No message found.")
-	(if (mew-virtual-p)
-	    (save-excursion
-	      (goto-char (point-min))
-	      (if (re-search-forward
-		   (concat "\r " (regexp-quote fld) " " (car range) "\\([^0-9]\\|$\\)")
-		   nil t)
-		  (progn
-		    (setq fld (buffer-name))
-		    (goto-char (match-beginning 0))
-		    (beginning-of-line)
-		    (looking-at "^ *\\([1-9][0-9]*\\)[^0-9]")
-		    (setq range (list (mew-match 1)))))))
-	(if (listp (car range))
-	    (setq fld (car (car range))
-		  mess (car (cdr (car range))))
-	  (setq mess (car range)))
-	(mew-nmz-goto-folder-msg fld mess)
-	(message "Searching %s ... %s/%s." message fld mess)))))
+		()
+	      ;; all folder search
+	      (message "Searching %s...all folders" message)
+	      (setq range (mew-nmz-multi-pick
+			   (mew-nmz-expand-folder-regexp "*")
+			   (car pattern) t))
+	      (if (null range)
+		  (setq pattern (cdr pattern))
+		(setq fld (car (car range)))
+		(setq range (cdr (car range)))
+		(if (not child) (setq range (nreverse range)))
+		))))
+	(if (null range)
+	    (message "No message found.")
+	  (if (mew-virtual-p)
+	      (save-excursion
+		(goto-char (point-min))
+		(if (re-search-forward
+		     (concat " \006 " (regexp-quote fld) " " (car range) "$")
+		     nil t)
+		    (progn
+		      (setq fld (buffer-name))
+		      (goto-char (match-beginning 0))
+		      (beginning-of-line)
+		      (looking-at "^ *\\([1-9][0-9]*\\)[^0-9]")
+		      (setq range (list (match-string 1)))))))
+	  (if (listp (car range))
+	      (setq fld (car (car range))
+		    mess (car (cdr (car range))))
+	    (setq mess (car range)))
+	  (mew-nmz-goto-folder-msg fld mess)
+	  (message "Searching %s...%s/%s" message fld mess))))))
 
 (defun mew-nmz-search-msgid-at-point ()
   (interactive)
@@ -568,7 +570,7 @@ If executed with 'C-u', search *child* message."
   (if (string-match "<\\([^>]+\\)>" mid)
       (let ((mew-window-use-full t)
 	    (mew-use-full-window t))
-	(mew-nmz-search-parent nil (mew-match 1 mid)))
+	(mew-nmz-search-parent nil (concat "\"" (match-string 1 mid) "\"")))
     (message "No Message-ID.")))
 
 ;;
@@ -585,16 +587,16 @@ then put the '*' mark onto them. "
 	   pattern msgs msgsback threadmsgs total linenum msgtotal)
        (mew-sinfo-set-find-key nil) ;; force to ask a user
        (setq pattern (mew-nmz-input-pick-pattern))
-       (message "Namazu picking messages in %s ..." fld)
+       (message "Namazu picking messages in %s..." fld)
        (save-excursion
 	 (setq msgs (cdr (car (mew-nmz-multi-pick
 			       (list (mew-nmz-expand-folder fld)) pattern))))
-	 (message "Namazu picking messages in %s ... done" fld)
+	 (message "Namazu picking messages in %s...done" fld)
 	 (if (null msgs)
 	     (message "No message to be marked.")
 	   (setq msgsback msgs)
 	   (setq msgtotal (length msgs))
-	   (message "Marking messages ...  0/%d" msgtotal)
+	   (message "Marking messages... 0/%d" msgtotal)
 	   ;; Thread folder
 	   (when (mew-thread-p)
 	     (setq threadmsgs (mew-summary-thread-get-msglst
@@ -611,7 +613,7 @@ then put the '*' mark onto them. "
 		 (mew-summary-mark-as mew-mark-review))
 	       (setq i (1+ i))
 	       (when (= (% i 50) 0)
-		 (message "Marking messages ... %d/%d" i msgtotal))
+		 (message "Marking messages...%d/%d" i msgtotal))
 	       (setq msgs (cdr msgs)))
 	     (set-buffer-modified-p nil))
 	   ;; Physical folder
@@ -627,10 +629,10 @@ then put the '*' mark onto them. "
 		   (forward-line))
 		 (setq i (1+ i))
 		 (when (= (% i 50) 0)
-		   (message "Marking messages ... %d/%d" i msgtotal))
+		   (message "Marking messages...%d/%d" i msgtotal))
 		 (setq msgs (cdr msgs)))
 	       (set-buffer-modified-p nil)))
-	   (message "Marking messages ... %d/%d done" i msgtotal)))))))
+	   (message "Marking messages...%d/%d done" i msgtotal)))))))
 
 ;;
 ;; "Namazu virtual" function.
@@ -640,18 +642,19 @@ then put the '*' mark onto them. "
   (if (not (mew-summary-or-virtual-p))
       (message "This command cannot be used in this mode.")
     (let ((vfld (concat
-		 "++"
+		 "*"
 		 (mew-input-string "Namazu virtual folder name %s(%s): "
 				   "" ;; dummy
 				   "vnamazu")))
-	  (flds (mew-input-folders (concat (mew-summary-folder-name) "*")))
+	  (flds (mew-input-refile-folders (list (concat (mew-summary-folder-name) "*"))
+					  t nil "+"))
 	  (pattern (mew-nmz-input-pick-pattern))
 	  (buf (generate-new-buffer mew-buffer-prefix))
 	  (file (mew-make-temp-name))
-	  nmzdirs fldmsgs fld msgs fasts scans count)
+	  nmzdirs fldmsgs fld msgs fasts scans count func lra)
       (if (null mew-nmz-indexed-folders)
 	  (mew-nmz-gather-indexed-folder))
-      (setq nmzdirs (mew-nmz-flds-to-indexs flds))
+      (setq nmzdirs (mew-nmz-flds-to-indexs (or flds '("+*"))))
       (if (null nmzdirs)
 	  (message "Please make namazu index.")
 	(setq fldmsgs (mew-nmz-multi-pick nmzdirs pattern))
@@ -669,10 +672,9 @@ then put the '*' mark onto them. "
 	  (mew-summary-switch-to-folder vfld)
 	  (mew-erase-buffer)
 	  (setq count 1)
-	  (if (null fasts)
-	      ()
+	  (when fasts
 	    (while (setq fld (car (car fasts)))
-	      (message "Namazu picking ... (%s)" fld)
+	      (message "Namazu picking...(%s)" fld)
 	      (setq pattern "")
 	      (setq msgs (cdr (car fasts)))
 	      (set-buffer buf)
@@ -684,29 +686,19 @@ then put the '*' mark onto them. "
 		(expand-file-name mew-summary-cache-file (mew-expand-folder fld))))
 	      (goto-char (point-min))
 	      (while (and (not (eobp)) msgs)
-		(if (not (re-search-forward (concat "^ *" (car msgs) "[^0-9]") nil t))
-		    ()
+		(when (re-search-forward (concat "^ *" (car msgs) "[^0-9]") nil t)
 		  (beginning-of-line)
-		  (cond
-		   ((looking-at	;; new format "\r <msgid> <par-id>"
-		     "^ *\\([1-9][0-9]*\\)[^0-9]\\([^\r]*\\)\r\\([^\n]*\\)")
+		  (when (looking-at	;; new format "\r <msgid> <par-id> \006 folder msg"
+			 "^ *\\([1-9][0-9]*\\)[^0-9]\\([^\r]*\r\\)\\([^\n]*\\)")
 		    (setq pattern
+			  ;; 123 .... \r  <msgid> <par-id> folder num
 			  (concat pattern
-				  (format "%5d %s\r %s %s%s\n"
+				  (format "%5d %s%s \006 %s %s\n"
 					  count
-					  (mew-match 2)
-					  fld (car msgs)
-					  (mew-match 3))))
-		    (setq count (1+ count)))
-		   ((looking-at	;; old format
-		     "^ *\\([1-9][0-9]*\\)[^0-9]\\(.*\\)$")
-		    (setq pattern
-			  (concat pattern
-				  (format "%5d %s\r %s %s\n"
-					  count
-					  (mew-match 2)
+					  (match-string 2)	;; don't use match-string
+					  (match-string 3)	;; for fancy-hightlight
 					  fld (car msgs))))
-		    (setq count (1+ count)))))
+		    (setq count (1+ count))))
 		(setq msgs (cdr msgs))
 		(forward-line 1))
 	      (mew-summary-switch-to-folder vfld)
@@ -715,11 +707,12 @@ then put the '*' mark onto them. "
 	       (insert pattern))
 	      (setq fasts (cdr fasts))))
 	  (if (null scans)
-	      (message "Namazu picking ... done.")
+	      (message "Namazu picking...done")
 	    (set-buffer buf)
 	    (buffer-disable-undo (current-buffer))
 	    (mew-erase-buffer)
 	    (while (setq fld (car (car scans)))
+	      (setq lra (cons (cons fld fld) lra))
 	      (insert "CD: " fld "\n")
 	      (setq msgs (cdr (car scans)))
 	      (while msgs
@@ -732,13 +725,10 @@ then put the '*' mark onto them. "
 	    (setq scans t))
 	  (mew-remove-buffer buf)
 	  (mew-summary-switch-to-folder vfld)
-	  (mew-vinfo-set-count count)
-	  (if (null scans)
-	      () ;; finish
-	    (mew-vinfo-set-func
-	     (` (lambda ()
-		  (if (file-exists-p (, file)) (delete-file (, file))))))
-	    (mew-scan (list "-i" file) nil 'virtual)))))))
+	  (when scans
+	    (setq func `(lambda () (when (file-exists-p ,file)
+				     (delete-file ,file))))
+	    (mew-local-retrieve 'vir (list "-i" file) count func lra 'noerase)))))))
 
 ;;
 ;; Use namazu-mode.
@@ -763,7 +753,7 @@ then put the '*' mark onto them. "
 	 (let ((flds (mew-input-folders (concat (mew-summary-folder-name) "*")))
 	       (pattern (mew-nmz-input-pick-pattern))
 	       fld nmzdirs dirlen)
-	   (setq nmzdirs (mew-nmz-flds-to-indexs flds))
+	   (setq nmzdirs (mew-nmz-flds-to-indexs (or flds '("+*"))))
 	   (setq dirlen (length nmzdirs))
 	   (if (not (> dirlen mew-nmz-db-max))
 	       ()
@@ -787,8 +777,8 @@ then put the '*' mark onto them. "
     (beginning-of-line)
     (if (not (re-search-forward pattern nil t))
 	(namazu-view)
-      (setq fld (mew-match 1))
-      (setq msg (mew-match 2))
+      (setq fld (match-string 1))
+      (setq msg (match-string 2))
       (setq fld (mew-nmz-url-to-folder fld))
       (beginning-of-line)
       (mew-nmz-goto-folder-msg fld msg))))
@@ -796,7 +786,7 @@ then put the '*' mark onto them. "
 (defun mew-nmz-namazu-return-mew ()
   (interactive)
   (if mew-nmz-namazu-last-folder
-      (mew-summary-goto-folder nil mew-nmz-namazu-last-folder)))
+      (mew-summary-visit-folder mew-nmz-namazu-last-folder nil 'no-ls)))
 
 (defun mew-nmz-namazu-goto-namazu ()
   (if (not (and (boundp 'namazu-buffer)
@@ -823,15 +813,6 @@ then put the '*' mark onto them. "
 			   "|"
 			   (substring (expand-file-name mew-mail-path) 2)))
 		 "\\|~/Mail\\)/.*/[1-9][0-9]*"))
-	(mew-news-pattern
-	 (concat "^\\("
-		 (if (not mew-nmz-use-backslash)
-		     (expand-file-name mew-news-path)
-		   (concat "/"
-			   (substring (expand-file-name mew-news-path) 0 1)
-			   "|"
-			   (substring (expand-file-name mew-news-path) 2)))
-		 "\\|~/News\\)/.*/[1-9][0-9]*"))
 	pos-title pos-url)
     (end-of-line)
     (and (re-search-backward namazu-output-title-pattern nil t)
@@ -850,8 +831,7 @@ then put the '*' mark onto them. "
     (goto-char pos)
     (cond
      ((and pos-title pos-url
-	   (or (looking-at mew-mail-pattern)
-	       (looking-at mew-news-pattern)))
+	   (looking-at mew-mail-pattern))
       (mew-nmz-namazu-goto-mew))
      ((and pos-title pos-url)
       (namazu-view))
@@ -870,7 +850,7 @@ then put the '*' mark onto them. "
     (setq mew-input-complete-function (function mew-nmz-pick-pattern))
     (let ((keymap (copy-keymap mew-input-map)) pattern)
       (define-key keymap " " nil)
-      (setq pattern (read-from-minibuffer "Namazu pick pattern : "
+      (setq pattern (read-from-minibuffer "Namazu pick pattern: "
 					  mew-nmz-pick-default-field
 					  keymap
 					  nil
@@ -882,47 +862,48 @@ then put the '*' mark onto them. "
       pattern)))
 
 (defun mew-nmz-pick-pattern-gather-header ()
-  (if mew-nmz-pick-gather-field-list
-      (save-excursion
-	(let ((buf (mew-cache-hit
-		    (mew-summary-folder-name) (mew-summary-message-number)))
-	      (gathers mew-nmz-pick-gather-field-list)
-	      retlst gather header duplchk mid addrs addr prefix)
-	  (if (not (and buf (get-buffer buf) (buffer-name (get-buffer buf))))
-	      ()
-	    (set-buffer buf)
-	    (while gathers
-	      (setq gather (car gathers))
-	      (setq header (mew-header-get-value (car gather)))
-	      (if (and header (car (cdr gather)))
-		  (cond
-		   ((eq (car (cdr gather)) 'msgid)
-		    (while (and header (string-match "<\\([^>]+\\)>" header))
-		      (setq mid (mew-match 1 header))
-		      (setq header (substring header (match-end 0)))
-			(if (member mid duplchk)
-			    ()
-			  (setq prefix (nthcdr 2 gather))
-			  (setq duplchk (cons mid duplchk))
-			  (while prefix
-			    (setq retlst (cons (concat (car prefix) mid) retlst))
-			    (setq prefix (cdr prefix))))))
-		   ((eq (car (cdr gather)) 'address)
-		    (setq addrs (mew-addrstr-parse-address-list header))
-		    (while (setq addr (car addrs))
-		      (if (not (member addr duplchk))
-			  (let ((prefix (nthcdr 2 gather)))
-			    (setq duplchk (cons addr duplchk))
-			    (while prefix
-			      (setq retlst (cons (concat (car prefix) addr) retlst))
-			      (setq prefix (cdr prefix)))))
-		      (setq addrs (cdr addrs))))))
-	      (setq gathers (cdr gathers)))
-	    (if retlst
-		(setq retlst (append retlst
-				     (list
-				      (concat " " (make-string (- (window-width) 10) ?-))))))
-	    (nreverse retlst))))))
+  (when mew-nmz-pick-gather-field-list
+    (save-excursion
+      (let ((buf (mew-cache-hit
+		  (mew-summary-folder-name) (mew-summary-message-number)))
+	    (gathers mew-nmz-pick-gather-field-list)
+	    retlst gather header duplchk mid addrs addr prefix)
+	(when (and buf (get-buffer buf) (buffer-name (get-buffer buf)))
+	  (set-buffer buf)
+	  (while gathers
+	    (setq gather (car gathers))
+	    (setq header (mew-header-get-value (car gather)))
+	    (when (and header (car (cdr gather)))
+	      (cond
+	       ((eq (car (cdr gather)) 'msgid)
+		(while (and header (string-match "<\\([^>]+\\)>" header))
+		  (setq mid (match-string 1 header))
+		  (setq header (substring header (match-end 0)))
+		  (if (member mid duplchk)
+		      ()
+		    (setq prefix (nthcdr 2 gather))
+		    (setq duplchk (cons mid duplchk))
+		    (while prefix
+		      (setq retlst (cons (concat (car prefix) mid) retlst))
+		      (setq prefix (cdr prefix))))))
+	       ((eq (car (cdr gather)) 'address)
+		(setq addrs (mew-addrstr-parse-address-list header))
+		(while (setq addr (car addrs))
+		  (setq addr (downcase addr))
+		  (if (not (member addr duplchk))
+		      (let ((prefix (nthcdr 2 gather)))
+			(setq duplchk (cons addr duplchk))
+			(while prefix
+			  (setq retlst (cons (concat (car prefix) addr) retlst))
+			  (setq prefix (cdr prefix)))))
+		  (setq addrs (cdr addrs))))))
+	    (setq gathers (cdr gathers)))
+	  (when retlst
+	    (setq retlst (append
+			  retlst
+			  (list
+			   (concat " " (make-string (- (window-width) 10) ?-))))))
+	  (nreverse retlst))))))
 
 (defun mew-nmz-pick-pattern ()
   (let* ((pat (mew-delete-pattern))
@@ -1018,11 +999,12 @@ then put the '*' mark onto them. "
     (mew-uniq-list nmzdirs)))
 
 (defun mew-nmz-goto-folder-msg (fld msg)
-  (mew-summary-goto-folder nil fld)
+  (mew-summary-visit-folder fld)
   (while (processp mew-summary-buffer-process)
     (sit-for 1)
     (discard-input))
   (mew-summary-jump-message msg)
+  (mew-summary-thread-move-cursor)
   (mew-summary-display 'force))
 
 (defun mew-nmz-make-temp-name (prefix)
@@ -1047,6 +1029,12 @@ then put the '*' mark onto them. "
 	  (setq pos (1+ (match-end 0))))
 	dir))))
 
+(defconst mew-nmz-url-to-folder-regex
+  (concat "^"
+	  (regexp-quote
+	   (file-name-as-directory (expand-file-name mew-mail-path)))
+	  "\\(.*\\)$"))
+
 (defun mew-nmz-url-to-folder (url)
   "Convert namazu's output url to folder."
   (and mew-nmz-use-backslash
@@ -1056,22 +1044,9 @@ then put the '*' mark onto them. "
 		  ":"
 		  (substring url (match-beginning 2) (match-end 2)))))
   (setq url (expand-file-name url))
-  (cond
-   ((string-match (concat
-		   "^"
-		   (regexp-quote
-		    (file-name-as-directory (expand-file-name mew-mail-path)))
-		   "\\(.*\\)$")
-		  url)
-    (concat "+" (substring url (match-beginning 1) (match-end 1))))
-   ((string-match (concat
-		   "^"
-		   (regexp-quote
-		    (file-name-as-directory (expand-file-name mew-news-path)))
-		   "\\(.*\\)$")
-		  url)
-    (concat "=" (substring url (match-beginning 1) (match-end 1))))
-   (t (message "Ignore url(%s) in %s." url "NMZ.field.uri"))))
+  (if (string-match mew-nmz-url-to-folder-regex url)
+      (concat "+" (substring url (match-beginning 1) (match-end 1)))
+    (message "Ignore url(%s) in %s." url "NMZ.field.uri")))
 
 (defun mew-nmz-index-newp (fld)
   (let ((touchtime (if mew-touch-folder-p
@@ -1097,7 +1072,8 @@ then put the '*' mark onto them. "
 (defun mew-nmz-folder-newp (fld)
   (let ((buf (get-buffer fld)))
     (if (not buf)
-	(let ((tdir (if mew-touch-folder-p
+	(let ((dir (file-chase-links (mew-expand-folder fld)))
+	      (tdir (if mew-touch-folder-p
 			(mew-file-get-time
 			 (expand-file-name mew-summary-touch-file
 					   (mew-expand-folder fld)))
@@ -1106,13 +1082,17 @@ then put the '*' mark onto them. "
 	      (tfile (mew-file-get-time
 		      (expand-file-name mew-summary-cache-file
 					(mew-expand-folder fld)))))
-	  (cond
-	   ((null tdir) t)
-	   ((null tfile) nil)
-	   ((> (car tdir) (car tfile)) nil)
-	   ((= (car tdir) (car tfile))
-	    (if (> (nth 1 tdir) (nth 1 tfile)) nil t)) ;; nil if same
-	   (t t)))
+	  (if (and mew-touch-folder-p (null tdir)
+		   (file-directory-p dir)
+		   (mew-dir-messages (mew-expand-folder dir)))
+	      nil
+	    (cond
+	     ((null tdir) t)
+	     ((null tfile) nil)
+	     ((> (car tdir) (car tfile)) nil)
+	     ((= (car tdir) (car tfile))
+	      (if (> (nth 1 tdir) (nth 1 tfile)) nil t)) ;; nil if same
+	     (t t))))
       (set-buffer buf)
       (if (and (not (mew-summary-folder-dir-newp))
 	       (mew-summary-exclusive-p))
@@ -1125,8 +1105,8 @@ then put the '*' mark onto them. "
   (let ((nmzflds mew-nmz-indexed-folders)
 	nmzdirs tmpfld)
     (if (not (string-match "^\\([^*]+\\)\\*?$" fld))
-	(setq fld "^[+=]")
-      (setq fld (mew-match 1 fld))
+	(setq fld "^\\+")
+      (setq fld (match-string 1 fld))
       (setq fld (concat (regexp-quote (directory-file-name fld)))))
     (while (setq tmpfld (car nmzflds))
       (and (string-match fld tmpfld)
@@ -1138,16 +1118,16 @@ then put the '*' mark onto them. "
 (defun mew-nmz-gather-indexed-folder ()
   "Gather indexed folder."
   (interactive)
-  (let ((flds mew-folder-list)
-	nmzdir nmzflds fld)
-    (while (setq fld (car flds))
-      (and (setq nmzdir (mew-nmz-expand-folder fld))
-	   (file-directory-p nmzdir)
-	   (file-exists-p (expand-file-name "NMZ.i" nmzdir))
-	   (setq nmzflds (cons (directory-file-name fld) nmzflds)))
+  (let ((flds (mapcar 'car mew-local-folder-alist))
+	nmzdir nmzflds)
+    (while flds
+      (when (and (setq nmzdir (mew-nmz-expand-folder (car flds)))
+		 (file-directory-p nmzdir)
+		 (file-exists-p (expand-file-name "NMZ.i" nmzdir)))
+	(setq nmzflds (cons (directory-file-name (car flds)) nmzflds)))
       (setq flds (cdr flds)))
     (setq mew-nmz-indexed-folders (nreverse nmzflds))
-    (and (interactive-p) (message "Gather indexed folder...done."))))
+    (and (interactive-p) (message "Gather indexed folder...done"))))
 
 (defun mew-nmz-index-delete (nmzdir)
   "Delete namazu index files."
@@ -1171,11 +1151,11 @@ then put the '*' mark onto them. "
       nil)))
 
 (defun mew-nmz-have-figure-folder (fld)
-  (let ((flds mew-folder-list)
+  (let ((flds mew-local-folder-alist)
 	(regex (concat "^" fld "/[1-9][0-9]*/?$")))
     (catch 'match
       (while flds
-	(if (string-match regex (car flds))
+	(if (string-match regex (car (car flds)))
 	    (throw 'match t))
 	(setq flds (cdr flds)))
       nil)))
