@@ -95,7 +95,7 @@
       (setq mew-alias-auto-alist (mew-lisp-load mew-alias-auto-file)))
   (setq mew-addrbook-orig-alist (mew-addrbook-make-alist))
   (mew-addrbook-concat-uniq)
-  (add-hook 'kill-emacs-hook (function mew-addrbook-clean-up)))
+  (add-hook 'kill-emacs-hook 'mew-addrbook-clean-up))
 
 (defun mew-addrbook-concat-uniq ()
   (setq mew-addrbook-alist
@@ -103,7 +103,7 @@
   (setq mew-addrbook-alist (mew-uniq-alist mew-addrbook-alist)))
 
 (defun mew-addrbook-clean-up ()
-  (remove-hook 'kill-emacs-hook (function mew-addrbook-clean-up))
+  (remove-hook 'kill-emacs-hook 'mew-addrbook-clean-up)
   (mew-lisp-save mew-alias-auto-file mew-alias-auto-alist)
   (setq mew-alias-auto-alist nil))
 
@@ -119,8 +119,8 @@
 (defun mew-addrbook-alias-get (key alist)
   (let* ((mew-alias-expand-prefix nil)
 	 (addrs (mew-alias-expand key alist 0))
-	 (addrx (mapcar (function mew-addrstr-append-domain) addrs))
-	 (ret (mapconcat (function identity) addrx ", ")))
+	 (addrx (mapcar 'mew-addrstr-append-domain addrs))
+	 (ret (mapconcat 'identity addrx ", ")))
     (if mew-alias-expand-prefix
 	(concat mew-alias-expand-prefix ":" ret ";")
       ret)))
@@ -128,7 +128,7 @@
 (defun mew-alias-expand-addrs (key alist count)
   (let ((keys (mew-split key ?,))
 	ret)
-    (setq keys (mapcar (function mew-chop) keys))
+    (setq keys (mapcar 'mew-chop keys))
     (setq keys (delete "" keys))
     (while keys
       (setq ret (nconc ret (mew-alias-expand (car keys) alist count)))
@@ -146,8 +146,8 @@ returned."
    ((> count mew-expand-max-depth) key)
    ((string-match "^\\([^:]+\\):\\([^;]+\\);$" key)
     (if mew-alias-expand-prefix (error ":; must not recurse!"))
-    (setq mew-alias-expand-prefix (mew-match 1 key))
-    (setq key (mew-match 2 key))
+    (setq mew-alias-expand-prefix (match-string 1 key))
+    (setq key (match-string 2 key))
     (mew-alias-expand-addrs key alist (1+ count)))
    (t
     (let ((crnt (mew-adbkent-addrs (mew-addrbook-alias-hit key alist))))
@@ -240,7 +240,7 @@ returned."
 	(setq path (file-name-directory par))
 	;; include children files
 	(while (re-search-forward "^<[ \t]*\\([^ \t\n]+\\).*$" nil t)
-	  (setq chr (expand-file-name (mew-match 1) path))
+	  (setq chr (expand-file-name (mew-match-string 1) path))
 	  (replace-match "" nil t)
 	  (when (and (file-readable-p chr) (not (member chr files)))
 	    (insert-file-contents chr)
@@ -311,17 +311,16 @@ returned."
 	     mew-addrbook-file mew-addrbook-comment-regex 'unquote)
 	(goto-char (point-min))
 	(while (re-search-forward "^ ?\\([^ \n:]+\\) ?\\(:?\\) ?\\([^ \n]+\\)" nil t)
-	  (setq alias (mew-addrbook-strsafe (mew-match 1)))
-	  (setq colon (mew-match 2))
-	  (setq addrs (mew-addrbook-strsafe (mew-match 3)))
+	  (setq alias (mew-addrbook-strsafe (mew-match-string 1)))
+	  (setq colon (mew-match-string 2))
+	  (setq addrs (mew-addrbook-strsafe (mew-match-string 3)))
 	  (if (string= colon ":")
 	      (setq alist (cons (list alias addrs) alist))
-	    (and addrs (setq addrs (mapcar (function mew-chop) 
-					   (mew-split addrs ?,))))
+	    (and addrs (setq addrs (mapcar 'mew-chop (mew-split addrs ?,))))
 	    (if (looking-at " ?\\([^ \n]*\\) ?\\([^ \n]*\\)")
 		(progn
-		  (setq nick (mew-addrbook-strsafe (mew-match 1)))
-		  (setq name (mew-addrbook-strsafe (mew-match 2))))
+		  (setq nick (mew-addrbook-strsafe (mew-match-string 1)))
+		  (setq name (mew-addrbook-strsafe (mew-match-string 2))))
 	      (setq nick nil)
 	      (setq name nil))
 	    (setq alist (cons (list alias addrs nick name) alist))))))
@@ -357,12 +356,12 @@ it will add an alias."
  	    (if personalinfo
  		(setq addrs (car (mew-header-parse-address-list
  				  (list mew-to:))))
-	      (setq addrs (mapconcat (function identity)
+	      (setq addrs (mapconcat 'identity
 				     (mew-header-parse-address-list
 				      (list mew-to: mew-cc:))
 				     ",")))
  	  (when (string-match "\\(.*\\)<.*>" from)
-	    (setq name (mew-match 1 from))
+	    (setq name (match-string 1 from))
 	    (setq name (mew-chop name)))
  	  (if mew-addrbook-strip-domainpart
  	      (setq shortname (downcase (mew-addrstr-extract-user addrs)))
@@ -521,15 +520,13 @@ The keys that are defined for this mode are:
   (interactive)
   (let ((file mew-alias-auto-file))
     (setq mew-alias-auto-alist
-	  (sort mew-alias-auto-alist
-		(function (lambda (x y) (string< (car x) (car y))))))
+	  (sort mew-alias-auto-alist (lambda (x y) (string< (car x) (car y)))))
     (mew-lisp-save mew-alias-auto-file mew-alias-auto-alist)
     (unless (file-name-absolute-p file)
       (setq file (expand-file-name file mew-conf-path)))
     (find-file file)
     (emacs-lisp-mode)
-    (add-hook 'local-write-file-hooks
-	      (function mew-summary-addrbook-read-buffer))))
+    (add-hook 'local-write-file-hooks 'mew-summary-addrbook-read-buffer)))
 
 (provide 'mew-addrbook)
 
