@@ -35,7 +35,7 @@
 (defvar mew-vinfo nil) ;; Virtual mode
 (defvar mew-xinfo nil) ;; Analyzed
 
-(mapcar (function make-variable-buffer-local)
+(mapcar 'make-variable-buffer-local
 	(list 'mew-summary-buffer-process
 	      'mew-summary-buffer-process-status
 	      'mew-summary-buffer-left-msgs
