@@ -85,6 +85,9 @@
   (let* ((fun (mew-header-decode-get-func b-or-q))
 	 (cs (mew-charset-to-cs charset))
 	 str)
+    (when (and mew-use-autoconv-when-unknown
+	       (not (mew-coding-system-p cs)))
+      (setq cs mew-cs-autoconv))
     (cond
      ((not (mew-coding-system-p cs))
       mew-error-unknown-charset)
@@ -92,6 +95,13 @@
       (setq str (funcall fun estr))
       (if (null str)
 	  (mew-header-decode-get-error-str b-or-q)
+	(when (and mew-decode-broken
+		   (not (eq cs mew-cs-autoconv))
+		   (or (null charset)
+		       (mew-case-equal charset mew-us-ascii)
+		       (mew-case-equal charset mew-utf-8))
+		   (string-match mew-regex-singlebyte-nonascii str))
+	  (setq cs mew-cs-autoconv))
 	(if (null cs)
 	    str
 	  (mew-cs-decode-string str cs))))
@@ -208,7 +218,7 @@
 
 (defun mew-header-encode-string (str &optional key-len)
   (let* ((ecsdb (or (mew-ecsdb-guess-string str)
-		    (mew-charset-to-ecsdb mew-charset-m17n)))
+		    (mew-charset-to-ecsdb (mew-charset-m17n))))
 	 (hcs (mew-ecsdb-hcs ecsdb))
 	 charset b-or-q fun)
     (if (not (mew-coding-system-p hcs))
@@ -252,7 +262,7 @@
     (nreverse ret)))
 
 (defun mew-header-encode-comma-text (str)
-  (let ((str-list (mapcar (function mew-chop) (mew-split str ?,))))
+  (let ((str-list (mapcar 'mew-chop (mew-split str ?,))))
     (mew-header-encode-text (car str-list))
     (setq str-list (cdr str-list))
     (while str-list
@@ -473,7 +483,7 @@
 	    (setq e-list (mew-header-encode-string (car str-list) key-len))
 	    (setq head-is-e nil))
 	(setq e-list (mew-header-encode-string (car str-list))))
-      (insert (mapconcat (function identity) e-list " "))
+      (insert (mapconcat 'identity e-list " "))
       ;; ascii
       (setq str-list (cdr str-list))
       (when (car str-list)
@@ -526,7 +536,7 @@
 	(setq start (point))
 	(if (not (looking-at mew-keyval))
 	    (forward-line)
-	  (setq key (mew-match 1))
+	  (setq key (mew-match-string 1))
 	  (setq med (match-end 0))
 	  (setq type (mew-field-type-for-encoding key))
 	  (setq fast-path t)
@@ -598,7 +608,9 @@
 (defsubst mew-header-decode-string (key)
   (let ((beg (match-beginning 0)) 
 	(end (match-end 0))
-	(css (mew-header-decode (mew-match 1) (mew-match 2) (mew-match 3))))
+	(css (mew-header-decode (mew-match-string 1)
+				(mew-match-string 2)
+				(mew-match-string 3))))
     (setq css (mew-header-sanity-check-string2 css key))
     (delete-region beg end)
     (insert css)))
@@ -616,7 +628,7 @@
       (format "Charset (%s) for body is not supported.\n" msg))
      ((eq err 'no-charset)
       (if msg
-	  "Charset (us-ascii) for body is mis-matched.\n"
+	  (format "Charset (%s) for body is mis-matched.\n" msg)
 	"Charset for body is not specified.\n"))
      ((eq err 'multi)
       (format "Illegal encoding (%s) for multipart.\n" msg)))
@@ -834,12 +846,22 @@ That is, each line may be more than 75. "
 	    (setq num (1+ num))))
 	(when charset
 	  (setq cs (mew-charset-to-cs charset))
+	  (when (and mew-use-autoconv-when-unknown
+		     (not (mew-coding-system-p cs)))
+	    (setq cs mew-cs-autoconv))
+	  (when (and mew-decode-broken
+		     (not (eq cs mew-cs-autoconv))
+		     (or (null charset)
+			 (mew-case-equal charset mew-us-ascii)
+			 (mew-case-equal charset mew-utf-8))
+		     (string-match mew-regex-singlebyte-nonascii paramvalue))
+	    (setq cs mew-cs-autoconv))
 	  (if (mew-coding-system-p cs)
 	      (setq paramvalue (mew-cs-decode-string paramvalue cs))
 	    (setq paramvalue mew-error-unknown-charset)))
 	(setq ret (cons (list paramname paramvalue) ret))
 	(setq ext-sort (cdr ext-sort)))
-      (setq ret (mapcar (function mew-param-sanity-check) ret))
+      (setq ret (mapcar 'mew-param-sanity-check ret))
       (cons value (nreverse ret))))) ;; late return
 
 (defun mew-param-analyze (param)
@@ -847,10 +869,10 @@ That is, each line may be more than 75. "
           (paramname paramvalue section charset lang)."
   (let* (name section asterisk value charset lang)
     (when (string-match mew-header-param-regex param)
-      (setq name (mew-match 1 param))
-      (setq section (mew-match 2 param)) ;; *21
-      (setq asterisk (string= (mew-match 3 param) "*"))
-      (setq value (mew-match 4 param))
+      (setq name (match-string 1 param))
+      (setq section (match-string 2 param)) ;; *21
+      (setq asterisk (string= (match-string 3 param) "*"))
+      (setq value (match-string 4 param))
       (if (string= section "")
 	  (if asterisk
 	      (setq section 0)
@@ -867,10 +889,10 @@ That is, each line may be more than 75. "
 		(setq value (mew-param-analyze-broken value name))))
 	(when (and (= section 0)
 		   (string-match "^\\([^']*\\)'\\([^']*\\)'\\(.*\\)$" value))
-	  (setq charset (mew-match 1 value))
-	  (setq lang (mew-match 2 value))
+	  (setq charset (match-string 1 value))
+	  (setq lang (match-string 2 value))
 	  (if (string= lang "") (setq lang nil))
-	  (setq value (mew-match 3 value))
+	  (setq value (match-string 3 value))
 	  (if (string= value "") (setq value nil)))
 	(setq value (mew-q-decode-string value ?%)))
       (if section
@@ -891,9 +913,9 @@ That is, each line may be more than 75. "
 	;; So, pre/pst prevents infinite loop.
 	(setq pre (substring value 0 (match-beginning 0)))
 	(setq pst (substring value (match-end 0)))
-	(setq med (mew-header-decode (mew-match 1 value)
-				     (mew-match 2 value)
-				     (mew-match 3 value)))
+	(setq med (mew-header-decode (match-string 1 value)
+				     (match-string 2 value)
+				     (match-string 3 value)))
 	(if (string-match mew-header-decode-regex3 pst)
 	    (setq pst (substring pst (match-end 1))))
 	(setq value (concat pre med pst)))))
