@@ -17,6 +17,7 @@
 ;; symbol means decode with MIME encoding only.
 
 (defvar mew-dinfo-list '("limit" "decode-text" "decode-binary" "use-alt"))
+;; decode-text = t | no-cs-conv | no-cs-ld-conv
 
 (mew-blinfo-defun 'mew-dinfo mew-dinfo-list)
 
@@ -165,7 +166,7 @@
 	 (while (not (eobp))
 	   (if (not (looking-at mew-keyval))
 	       (forward-line)
-	     (setq key (mew-match 1))
+	     (setq key (mew-match-string 1))
 	     (setq beg (match-beginning 0))
 	     (setq med (match-end 0))
 	     (forward-line)
@@ -193,12 +194,12 @@ Return the existence MIME-Version: and the value of Subject:."
 	 (visibles (make-list (length mew-field-spec) nil))
 	 (prop mew-use-highlight-header)
 	 key beg med subj from contents others
-	 key-face val-face N nspec visiblep mimep)
+	 key-face val-face N nspec visiblep mimep fld)
     (mew-decode-narrow-to-header
      (while (not (eobp))
        (if (not (looking-at mew-keyval))
 	   (forward-line)
-	 (setq key (mew-capitalize (mew-match 1)))
+	 (setq key (mew-capitalize (mew-match-string 1)))
 	 (setq beg (match-beginning 0))
 	 (setq med (match-end 0))
 	 (forward-line)
@@ -212,7 +213,9 @@ Return the existence MIME-Version: and the value of Subject:."
 	   (delete-region beg (point)))
 	  ((string= key mew-x-mew-uidl:)
 	   (when (and (eq cnt 1)
-		      (not (mew-folder-mdropp (mew-cinfo-get-fld))))
+		      (setq fld (mew-cinfo-get-fld))
+		      (not (string= fld mew-trash-folder))
+		      (not (mew-folder-remotep (mew-folder-folder fld))))
 	     (mew-xinfo-set-action
 	      (substitute-command-keys 
 	       "Too large, truncated (the 'T' mark). To get the entire message, type '\\<mew-summary-mode-map>\\[mew-summary-retrieve-message]'"))))
@@ -270,21 +273,21 @@ Return the existence MIME-Version: and the value of Subject:."
     (put-text-property (point-min) (point) 'mew-noncontents t)
     (unless mew-field-other-visible
       (setq beg (point))
-      (mapcar (function insert) (nreverse others))
+      (mapcar 'insert (nreverse others))
       (put-text-property beg (point) 'mew-others t)
       (put-text-property beg (point) 'mew-noncontents t))
     (setq beg (point))
-    (mapcar (function (lambda (x) (and (stringp x) (insert x)))) visibles)
+    (mapcar (lambda (x) (and (stringp x) (insert x))) visibles)
     ;; for recenter in Message mode
     (put-text-property beg (point) 'mew-visible t)
     (put-text-property beg (point) 'mew-noncontents t)
     (when mew-field-other-visible
       (setq beg (point))
-      (mapcar (function insert) (nreverse others))
+      (mapcar 'insert (nreverse others))
       (put-text-property beg (point) 'mew-others t)
       (put-text-property beg (point) 'mew-noncontents t))
     ;; the beginning of the content header
-    (save-excursion (mapcar (function insert) (nreverse contents)))
+    (save-excursion (mapcar 'insert (nreverse contents)))
     ;; 'mew-contents does not work due to PGP/MIME
     (list mimep subj from)))
 
@@ -299,7 +302,7 @@ Return a part syntax after moving the beginning of the content body."
      (while (not (eobp))
        (if (not (looking-at mew-keyval))
 	   (forward-line)
-	 (setq key (mew-capitalize (mew-match 1)))
+	 (setq key (mew-capitalize (mew-match-string 1)))
 	 (setq med (match-end 0))
 	 (forward-line)
 	 (mew-header-goto-next)
@@ -324,7 +327,7 @@ Return a part syntax after moving the beginning of the content body."
 	    ((eq act 'id)
 	     (setq value (mew-buffer-substring med (1- (point))))
 	     (if (string-match mew-regex-id value)
-		 (setq value (mew-match 1 value))
+		 (setq value (match-string 1 value))
 	       (setq value nil))))
 	   (aset vec n value)))))
     (if (eobp)
@@ -433,7 +436,7 @@ Return a part syntax after moving the beginning of the content body."
 	    (forward-line)
 	  (mew-decode-error "No end-of-header(null line) in RFC822 message"))
 	;; the beginning of the BODY(i.e. the physical content body)
-	(if (eq (mew-dinfo-get-decode-text) t)
+	(if (eq (mew-dinfo-get-decode-text) t) ;; not no-cs-conv, no-cs-ld-conv
 	    (mew-cs-decode-region (point) (point-max) mew-cs-autoconv))
 	(mew-syntax-set-key syntax 'message)
 	(mew-syntax-set-end syntax (1- (point)))
@@ -457,10 +460,10 @@ Return a part syntax after moving the beginning of the content body."
 	    ;; Emacs 20
 	    (setq decoded (base64-decode-region beg (point-max)))
 	  (error (setq decoded nil))) ;; Emacs 21
-	(when (and decoded linebasep)
+	(when (and decoded linebasep
+		   (not (eq (mew-dinfo-get-decode-text) 'no-cs-ld-conv)))
 	  (goto-char beg)
-	  (while (search-forward "\r\n" nil t)
-	    (replace-match "\n" nil t))))
+	  (mew-crlf-to-lf)))
       ;; base64-decode-region may fail if garbage exists.
       (cond
        (decoded
@@ -484,13 +487,12 @@ Return a part syntax after moving the beginning of the content body."
 	   mew-cs-binary mew-cs-dummy
 	   ;; mew-prog-mime-decode converts CRLF to LF, so
 	   ;; read input as binary.
-	   (apply (function call-process) mew-prog-mime-decode
-		  file '(t nil) nil opt))
-	  (if (file-exists-p file) (delete-file file))))
+	   (apply 'call-process mew-prog-mime-decode file '(t nil) nil opt))
+	  (mew-delete-file file)))
        (t
 	(mew-decode-error (concat mew-prog-mime-decode " does not exist")))))
     ;; charset conversion
-    (if (and textp (eq (mew-dinfo-get-decode-text) t)) ;; not 'no-cs-conv
+    (if (and textp (eq (mew-dinfo-get-decode-text) t)) ;; not no-cs-conv, no-cs-ld-conv
 	(mew-decode-charset-conv charset beg))))
 
 (defun mew-decode-charset-conv (charset beg)
@@ -499,7 +501,11 @@ Return a part syntax after moving the beginning of the content body."
     ;; (mew-coding-system-p nil) returns t.
     (unless (mew-coding-system-p fromcs)
       (mew-decode-warning-body 'unknown-charset charset))
+    (when (and mew-use-autoconv-when-unknown
+	       (not (mew-coding-system-p fromcs)))
+      (setq fromcs mew-cs-autoconv))
     (when (and mew-decode-broken
+	       (not (eq fromcs mew-cs-autoconv))
 	       (or (null charset)
 		   (mew-case-equal charset mew-us-ascii))) ;; anyway
       (goto-char beg)
@@ -507,6 +513,16 @@ Return a part syntax after moving the beginning of the content body."
 	(mew-decode-warning-body 'no-charset charset)
 	(setq fromcs mew-cs-autoconv))
       (goto-char beg))
+    (when (and mew-decode-broken
+	       (not (eq fromcs mew-cs-autoconv))
+	       (or (null charset)
+		   (mew-case-equal charset mew-us-ascii)
+		   (mew-case-equal charset mew-utf-8)))
+      (goto-char beg)
+      (when (re-search-forward mew-regex-singlebyte-nonascii nil t)
+	(mew-decode-warning-body 'no-charset charset)
+	(setq fromcs mew-cs-autoconv))
+      (goto-char beg))
     (if (mew-coding-system-p fromcs)
 	(mew-cs-decode-region beg (point-max) fromcs))))
 
@@ -564,7 +580,7 @@ Return a part syntax after moving the beginning of the content body."
 	 ((string= mew-ct-mls ct)
 	  (setq syntax (mew-decode-multipart-signed syntax cnt)))
 	 ((string= mew-ct-mle ct)
-	  (if (boundp 'mew-inherit-prefetching) ;; xxx
+	  (if mew-inherit-prefetching
 	      (signal 'quit "")
 	    (setq syntax (mew-decode-multipart-encrypted syntax cnt))))
 	 (t
@@ -573,6 +589,12 @@ Return a part syntax after moving the beginning of the content body."
      ((string-match mew-ct-sme ct)
       (setq syntax (mew-smime-decrypt-or-verify syntax cnt ctl cte)))
      ;; Others
+     ((string= mew-ct-trh ct)
+      (save-restriction
+	;; Eliminating the content the header so that Rfc822 header
+	;; comes the beginning.
+	(narrow-to-region (point) (point-max))
+	(mew-decode-rfc822-header)))
      (t
       (if (and (eq parent 'message)
 	       (or (not (mew-xinfo-get-text-body)) (not textp)))
@@ -619,10 +641,10 @@ Return a part syntax after moving the beginning of the content body."
   (concat "^--" (regexp-quote boundary) "\\(--\\|\\)[ \t]*$"))
 
 (defsubst mew-decode-multipart-boundary-cont ()
-  (string= (mew-match 1) ""))
+  (string= (mew-match-string 1) ""))
 
 (defsubst mew-decode-multipart-boundary-end ()
-  (string= (mew-match 1) "--"))
+  (string= (mew-match-string 1) "--"))
 
 (defun mew-decode-multipart (syntax cnt &optional dct)
   (let* ((case-fold-search nil) ;; boundary is case sensitive
@@ -777,9 +799,9 @@ Return a part syntax after moving the beginning of the content body."
       (insert "Multipart/Encrypted could not be decrypted.\n")
       (mew-xinfo-set-not-decrypted t))
     ;; Throw away garbage
-    (and file1 (file-exists-p file1) (delete-file file1))
-    (and file2 (file-exists-p file2) (delete-file file2))
-    (and file3 (file-exists-p file3) (delete-file file3))
+    (mew-delete-file file1)
+    (mew-delete-file file2)
+    (mew-delete-file file3)
     ;; Analyze the decrypted part
     (goto-char (point-min))
     (setq syntax3 (mew-decode-singlepart cnt nil nil))
@@ -850,8 +872,8 @@ Return a part syntax after moving the beginning of the content body."
      (t
       (setq result (funcall func file1 file2))))
     ;; Throw away garbage
-    (and file1 (file-exists-p file1) (delete-file file1))
-    (and file2 (file-exists-p file2) (delete-file file2))
+    (mew-delete-file file1)
+    (mew-delete-file file2)
     ;; Analyze the signed part
     (goto-char (point-min))
     (setq syntax3 (mew-decode-singlepart cnt nil nil))
@@ -885,7 +907,7 @@ Return a part syntax after moving the beginning of the content body."
       (catch 'header
 	(while (re-search-forward 
 		"^\r?$\\|^Content-Transfer-Encoding:[ \t]*" nil t)
-	  (setq key (mew-match 0))
+	  (setq key (mew-match-string 0))
 	  (setq start (match-end 0))
 	  (when (string-match "^\r?$" key)
 	    (save-restriction
@@ -893,8 +915,7 @@ Return a part syntax after moving the beginning of the content body."
 		  (narrow-to-region (point-min) (1+ start))
 		(narrow-to-region (point-min) (point-max)))
 	      (goto-char (point-min))
-	      (while (search-forward "\r\n" nil t)
-		(replace-match "\n" nil t)))
+	      (mew-crlf-to-lf))
 	    (throw 'header nil))
 	  (forward-line)
 	  (mew-header-goto-next)
