@@ -26,7 +26,7 @@ Execute mew-dinfo-set before calling this."
 	(error
 	 (error "MIME decoding error: %s" (mew-xinfo-get-decode-err)))))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Header mode
 ;;;
@@ -87,55 +87,97 @@ Execute mew-dinfo-set before calling this."
 (defun mew-header-process-message (action)
   (if (not (file-readable-p (mew-tinfo-get-hdr-file)))
       (message "No corresponding message!")
-    (let* ((buf (current-buffer))
-	   (case (mew-tinfo-get-case))
-	   (server (mew-smtp-server case))
-	   (ssh-server (mew-smtp-ssh-server case))
-	   (pnm (mew-smtp-info-name case))
-	   sendit resentp fcc err)
-      (if (mew-smtp-get-lock pnm)
-	  (message "Another message is being sent. Try later")
-	(goto-char (point-min))
-	(setq resentp (re-search-forward "^Resent" (mew-header-end) t))
-	(setq fcc (mew-encode-ask-fcc resentp))
-	(if (and (eq action 'send) mew-ask-send)
-	    (setq sendit (y-or-n-p "Really send this message? "))
-	  (setq sendit t))
-	(when sendit
-	  (mew-smtp-set-case pnm case)
-	  (mew-smtp-set-server pnm server)
-	  (mew-smtp-set-port pnm (mew-smtp-port case))
-	  (mew-smtp-set-ssh-server pnm ssh-server)
-	  (mew-smtp-set-queue pnm (mew-queue-folder case))
-	  (mew-smtp-set-fcc pnm fcc)
-	  (mew-current-get-window-config)
-	  (mew-redraw)
-	  (save-excursion
-	    (save-window-excursion
-	      (set-buffer buf)
-	      (mew-encode-make-backup)
-	      (mew-header-clear)
-	      ;; the end of the header
-	      (save-restriction
-		(narrow-to-region (point) (point))
-		(mew-frwlet
-		 mew-cs-text-for-read mew-cs-text-for-write
-		 (insert-file-contents (mew-tinfo-get-hdr-file)))
-		(goto-char (point-min))
-		(mew-header-goto-end)
-		(mew-header-set mew-header-separator)
-		(mew-header-delete-lines mew-field-delete-common)
-		(mew-header-delete-lines (mew-tinfo-get-field-del)))
-	      (if (mew-encode pnm case resentp nil nil 'header)
-		  (cond
-		   ((eq action 'queue)
-		    (mew-smtp-queue pnm "from Draft mode"))
-		   ((eq action 'send)
-		    (mew-smtp-send-message pnm)))
-		(setq err t))))
-	  (when err
-	    (switch-to-buffer buf)
-	    (delete-other-windows)))))))
+    (if (mew-header-existp mew-newsgroups:)
+	(mew-header-nntp-process-message action)
+      (mew-header-smtp-process-message action))))
+
+(defun mew-header-smtp-process-message (action)
+  (let* ((buf (current-buffer))
+	 (case (mew-tinfo-get-case))
+	 (pnm (mew-smtp-info-name case))
+	 sendit resentp fcc err)
+    (if (get-process pnm)
+	(message "Another message is being sent. Try later")
+      (goto-char (point-min))
+      (setq resentp (mew-draft-resent-p (mew-header-end)))
+      (setq fcc (mew-encode-ask-fcc resentp))
+      (if (and (eq action 'send) mew-ask-send)
+	  (setq sendit (y-or-n-p "Really send this message? "))
+	(setq sendit t))
+      (when sendit
+	(mew-current-get-window-config)
+	(mew-redraw)
+	(save-excursion
+	  (save-window-excursion
+	    (set-buffer buf)
+	    (mew-encode-make-backup)
+	    (mew-header-clear)
+	    ;; the end of the header
+	    (save-restriction
+	      (narrow-to-region (point) (point))
+	      (mew-frwlet
+	       mew-cs-text-for-read mew-cs-text-for-write
+	       (insert-file-contents (mew-tinfo-get-hdr-file)))
+	      (goto-char (point-min))
+	      (mew-header-goto-end)
+	      (mew-header-set mew-header-separator)
+	      (mew-header-delete-lines mew-field-delete-common)
+	      (mew-header-delete-lines (mew-tinfo-get-field-del)))
+	    (if (mew-smtp-encode pnm case resentp fcc nil nil 'header)
+		(cond
+		 ((eq action 'queue)
+		  (mew-smtp-queue case "from Draft mode"))
+		 ((eq action 'send)
+		  (mew-smtp-send-message case)))
+	      (setq err t))))
+	(when err
+	  (mew-current-set-window-config)
+	  (switch-to-buffer buf)
+	  (delete-other-windows))))))
+
+(defun mew-header-nntp-process-message (action)
+  (let* ((buf (current-buffer))
+	 (case (mew-tinfo-get-case))
+	 (pnm (mew-nntp2-info-name case))
+	 sendit resentp fcc err)
+    (if (get-process pnm)
+	(message "Another message is being posted. Try later")
+      (goto-char (point-min))
+      (setq resentp (mew-draft-resent-p (mew-header-end)))
+      (setq fcc (mew-encode-ask-fcc resentp))
+      (if (and (eq action 'send) mew-ask-send)
+	  (setq sendit (y-or-n-p "Really post this message? "))
+	(setq sendit t))
+      (when sendit
+	(mew-current-get-window-config)
+	(mew-redraw)
+	(save-excursion
+	  (save-window-excursion
+	    (set-buffer buf)
+	    (mew-encode-make-backup)
+	    (mew-header-clear)
+	    ;; the end of the header
+	    (save-restriction
+	      (narrow-to-region (point) (point))
+	      (mew-frwlet
+	       mew-cs-text-for-read mew-cs-text-for-write
+	       (insert-file-contents (mew-tinfo-get-hdr-file)))
+	      (goto-char (point-min))
+	      (mew-header-goto-end)
+	      (mew-header-set mew-header-separator)
+	      (mew-header-delete-lines mew-field-delete-common)
+	      (mew-header-delete-lines (mew-tinfo-get-field-del)))
+	    (if (mew-nntp2-encode pnm case fcc nil nil 'header)
+		(cond
+		 ((eq action 'queue)
+		  (mew-nntp2-queue case "from Draft mode"))
+		 ((eq action 'send)
+		  (mew-nntp2-send-message case)))
+	      (setq err t))))
+	(when err
+	  (mew-current-set-window-config)
+	  (switch-to-buffer buf)
+	  (delete-other-windows))))))
 
 (defun mew-header-make-message ()
   (interactive)
@@ -145,7 +187,7 @@ Execute mew-dinfo-set before calling this."
   (interactive)
   (mew-header-process-message 'send))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Saving
 ;;;
@@ -155,7 +197,10 @@ Execute mew-dinfo-set before calling this."
 you want to save, the entire message or its body. If the target is
 a non-message part, the part is saved (with line delimiter conversion
 if it is a text object). When executed with '\\[universal-argument]', coding-system for
-a text object to be saved can be specified."
+a text object to be saved can be specified.
+If 'binary' is specified to coding-system, the text object is treated as
+if it were binary. That is, no character set conversion nor line
+delimiter coversion occur."
   (interactive "P")
   (mew-summary-msg-or-part
    ;; need to make a cache or a message buffer.
@@ -176,6 +221,7 @@ a text object to be saved can be specified."
        ;; Due to mew-decode-broken, the filename may be changed.
        ;; So, save it here.
        (and syntax (setq cdpl (mew-syntax-get-cdp syntax)))
+       (setq ctl (mew-syntax-get-ct syntax))
        (setq file (mew-syntax-get-filename cdpl ctl))
        ;; First of all, we should determine which part the user want to
        ;; save due to the ambiguity.
@@ -210,7 +256,7 @@ a text object to be saved can be specified."
        (set-buffer buf)
        (mew-erase-buffer)
        (mew-insert-message fld msg mew-cs-text-for-read nil)
-       (mew-dinfo-set limit 'no-cs-conv t alt)
+       (mew-dinfo-set limit 'no-cs-ld-conv t alt)
        (mew-decode-for-edit)
        ;;
        (setq syntax (mew-syntax-get-entry mew-decode-syntax nums))
@@ -243,8 +289,10 @@ a text object to be saved can be specified."
 	     (setq ct (mew-syntax-get-value ctl 'cap))
 	     ;; Allow Message/Rfc822. It's user's risk.
 	     (setq linebasep (or (mew-ct-textp ct) (mew-ct-linebasep ct)))
-	     (if (and askcs linebasep mew-mule-p)
-		 (setq tocs (read-coding-system "Coding-system: ")))
+	     (when (and askcs linebasep mew-mule-p)
+	       (setq tocs (read-coding-system "Coding-system: "))
+	       (if (equal tocs mew-cs-binary)
+		   (setq tocs nil linebasep nil)))
 	     (when have-hdrp
 	       (goto-char (point-min))
 	       (mew-header-delete-lines mew-field-delete-common)
@@ -269,7 +317,7 @@ a text object to be saved can be specified."
 	      (write-region (point-min) (point-max) file append-p))))))
      (mew-remove-buffer buf))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Local cs
 ;;;
@@ -319,7 +367,7 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	 ;; Now, let's analyze the message in the burst buffer.
 	 ;; This is lengthy, though, avoidable.
 	 (setq limit (length nums))
-	 (if (or PLUS1P bodyp);; VERY important for PGP/MIME
+	 (if (or PLUS1P bodyp) ;; VERY important for PGP/MIME
 	     (setq limit (1+ limit)))
 	 ;;
 	 (set-buffer buf)
@@ -358,7 +406,7 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	    (mew-header-arrange (point-min) (point-max)))
 	  (setq start (point))
 	  (save-excursion
-	    (insert-buffer-substring buf beg end)
+	    (mew-insert-buffer-substring buf beg end)
 	    (mew-cs-decode-region start (point-max) tocs)
 	    (mew-highlight-body-region start (point-max)))
 	  ;; Page breaks
@@ -371,82 +419,97 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	 (select-window win))
        (mew-remove-buffer buf)))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Burst
 ;;;
 
-(defun mew-summary-burst-body (fld msg folder)
+(defun mew-summary-burst-body (fld msg folder &optional num)
   (save-excursion
     (let ((buf (generate-new-buffer mew-buffer-prefix))
-	  ret errmsg entry multi mstr m n len)
+	  ret errmsg mstr m)
       (set-buffer buf)
       (mew-erase-buffer)
       (mew-insert-message fld msg mew-cs-text-for-read nil)
       (mew-dinfo-set 1 nil nil nil) ;; xxx
       (mew-decode-for-edit)
-      (setq multi (mew-syntax-get-part mew-decode-syntax))
       (cond
-       ((not (mew-syntax-multipart-p multi))
+       ((not (mew-syntax-multipart-p (mew-syntax-get-part mew-decode-syntax)))
 	(message "Cannot burst"))
-       ((not (mew-folder-check folder))
-	(setq errmsg (format "%s is wrong. Nothing was processed" folder)))
        ((not (setq mstr (mew-folder-new-message folder 'num-only)))
 	(setq errmsg (format "Error in %s. Nothing was processed" folder)))
        (t
-	(setq m (string-to-int mstr))
-	(setq len (- (length multi) mew-syntax-magic))
-	(setq n 1)
-	(while (<= n len)
-	  (setq entry (mew-syntax-get-entry mew-decode-syntax (list n)))
-	  (if (not
-	       (string=
-		(mew-syntax-get-value (mew-syntax-get-ct entry) 'cap)
-		mew-ct-msg))
-	      () ;; return value
-	    (mew-frwlet
-	     mew-cs-dummy mew-cs-text-for-write
-	     (write-region
-	      (mew-syntax-get-begin entry)
-	      ;; This is RFC 822 message.
-	      ;; So, body is a single text/plain.
-	      (mew-syntax-get-end (mew-syntax-get-part entry))
-	      (mew-expand-folder folder (int-to-string m))
-	      nil 'no-msg))
-	    (setq m (1+ m)))
-	  (setq n (1+ n)))
-	(mew-touch-folder folder)
-	(setq ret (list mstr (int-to-string (1- m))))))
+	(setq m (mew-summary-burst-part (mew-syntax-get-part mew-decode-syntax)
+					folder (string-to-int mstr) num))
+	(if (= m (string-to-int mstr))
+	    (message "Nothing was processed")
+	  (mew-touch-folder folder)
+	  (setq ret (list mstr (int-to-string (1- m)))))))
       (mew-remove-buffer buf)
       (if errmsg (error errmsg))
       ret)))
 
+(defun mew-summary-burst-part (part folder newmsg &optional num)
+  (let (n len nums entry file)
+    (setq n 1)
+    (cond
+     (num
+      (setq nums (mew-syntax-number-to-nums num))
+      (setq len 1))
+     (t
+      (setq nums (list n))
+      (setq len (- (length part) mew-syntax-magic))))
+    (while (<= n len)
+      (setq entry (mew-syntax-get-entry part nums))
+      (cond
+       ((mew-syntax-multipart-p entry)
+	(setq newmsg (mew-summary-burst-part entry folder newmsg)))
+       ((string= (mew-syntax-get-value (mew-syntax-get-ct entry) 'cap)
+		 mew-ct-msg)
+	(setq file (mew-expand-folder folder (int-to-string newmsg)))
+	(mew-frwlet
+	 mew-cs-dummy mew-cs-text-for-write
+	 (write-region
+	  (mew-syntax-get-begin entry)
+	  ;; This is RFC 822 message.
+	  ;; So, body is a single text/plain.
+	  (mew-syntax-get-end (mew-syntax-get-part entry))
+	  file nil 'no-msg))
+	(mew-set-file-modes file)
+	(setq newmsg (1+ newmsg))))
+      (setq n (1+ n))
+      (setq nums (list n)))
+    newmsg))
+
 (defvar mew-burst-last-folder nil)
 
 (defun mew-input-burst-folder ()
   (let (default)
      (if (and mew-use-burst-folder-history mew-burst-last-folder)
 	 (setq default mew-burst-last-folder)
-       (setq default (mew-inbox-folder)))
-     (setq mew-burst-last-folder (mew-input-folder default))
+       (setq default (mew-inbox-folder))) ;; local folder
+     (setq mew-burst-last-folder (mew-input-local-folder default))
      mew-burst-last-folder))
 
-(defun mew-summary-burst ()
-  "De-capsulate messages embedded in this message."
-  (interactive)
+(defun mew-summary-burst (&optional arg)
+  "De-capsulate messages embedded in this message.
+If called with '\\[universal-argument]', the current part is de-capsulated."
+  (interactive "P")
   (mew-summary-msg-or-part
    (let ((fld (mew-summary-folder-name))
 	 (msg (mew-summary-message-number2))
 	 (folder (mew-input-burst-folder))
+	 (num (if arg (mew-syntax-number)))
 	 ret)
-     (message "Bursting...")
-     (setq ret (mew-summary-burst-body fld msg folder))
-     (when ret
-       (message "Bursting...done")
-       (if (y-or-n-p (format "Go to %s? " folder))
-	   (mew-summary-goto-folder t folder))
-       (message "Messages from %s to %s were extracted in %s"
-		(nth 0 ret) (nth 1 ret) folder)))))
+     (when (and folder (mew-local-folder-check folder 'ask))
+       (message "Bursting...")
+       (setq ret (mew-summary-burst-body fld msg folder num))
+       (when ret
+	 (message "Bursting...done")
+	 (if (y-or-n-p (format "Go to %s? " folder))
+	     (mew-summary-visit-folder folder 'goend))
+	 (message "Messages from %s to %s were extracted in %s"
+		  (nth 0 ret) (nth 1 ret) folder))))))
 
 (defun mew-summary-burst-multi ()
   "De-capsulate messages embedded in the messages marked with '@'."
@@ -454,15 +517,17 @@ If executed with '\\[universal-argument]', coding-system is asked."
   (mew-summary-multi-msgs
    (let ((folder (mew-input-burst-folder))
 	 (targets FLD-MSG-LIST))
-     (message "Bursting...")
-     (while targets
-       (mew-summary-burst-body (car (car targets)) (cdr (car targets)) folder)
-       (setq targets (cdr targets)))
-     (message "Bursting...done")
-     (if (y-or-n-p (format "Go to %s? " folder))
-	 (mew-summary-goto-folder t folder)))))
+     (when (and folder (mew-local-folder-check folder 'ask))
+       (message "Bursting...")
+       (while targets
+	 (mew-summary-burst-body
+	  (car (car targets)) (cdr (car targets)) folder)
+	 (setq targets (cdr targets)))
+       (message "Bursting...done")
+       (if (y-or-n-p (format "Go to %s? " folder))
+	   (mew-summary-visit-folder folder 'goend))))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Old burst
 ;;;
@@ -511,9 +576,9 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	 (setq done t)))
      (mew-remove-buffer buf)
      (if (and done (y-or-n-p (format "Go to %s? " folder)))
-	 (mew-summary-goto-folder t folder)))))
+	 (mew-summary-visit-folder folder 'goend)))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Re-editing
 ;;;
@@ -523,11 +588,9 @@ If executed with '\\[universal-argument]', coding-system is asked."
 typically included in a MIME-encapsulated error message.
 
 1. In +draft, the message is just edited.
-2. In +queue, the message is moved to +draft and is edited.
+2. In +queue or +postq, the message is moved to +draft and is edited.
 3. Otherwise, the message is copied to +draft and is edited. 
 
-For +draft and +queue, the sending case revives.
-
 For the other folders, if mew-case-guess-when-prepared is 't', each
 fields of the original header is replaced according to a guessed
 sending case.  If called with '\\[universal-argument]', the original
@@ -542,7 +605,7 @@ See also mew-summary-edit-again."
 	 draftname)
      (cond
       ((string= fld mew-draft-folder)
-       (setq draftname (mew-draft-buffer-name (mew-expand-folder fld msg)))
+       (setq draftname (mew-path-to-folder (mew-expand-folder fld msg)))
        (if (get-buffer draftname)
 	   (progn
 	     (mew-current-set-window-config) ;; xxx necessary?
@@ -550,10 +613,7 @@ See also mew-summary-edit-again."
 	     (switch-to-buffer draftname))
 	 (mew-summary-reedit-for-draft fld msg)
 	 (run-hooks 'mew-draft-mode-reedit-draft-hook)))
-      ((and (mew-folder-queuep fld) (null part))
-       (mew-decode-syntax-delete)
-       (mew-mark-kill-line)
-       (set-buffer-modified-p nil)
+      ((and (or (mew-folder-queuep fld) (mew-folder-postqp fld)) (null part))
        (mew-summary-reedit-for-queue fld msg)
        (run-hooks 'mew-draft-mode-reedit-queue-hook))
       (t
@@ -570,7 +630,7 @@ See also mew-summary-edit-again."
 	 (mew-summary-reedit-for-message fld msg nil keep-hdr nil)
 	 (run-hooks 'mew-draft-mode-reedit-hook)))))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Header
 ;;;
@@ -605,7 +665,7 @@ See also mew-summary-edit-again."
      (mew-draft-replace-fields nil))
    (mew-summary-edit-header-common)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Body
 ;;;
@@ -637,6 +697,12 @@ Note that the message is not a valid MIME message."
 (defun mew-summary-reedit-for-queue (fld msg)
   "Edit a message in a folder other than the draft folder.
 The message is assumed to be a valid MIME message."
+  ;; Message mode will be invalid so hide it.
+  (mew-decode-syntax-delete)
+  (mew-mark-kill-line)
+  (set-buffer-modified-p nil)
+  ;; Need to delete Message window because the target message will be
+  ;; modified and its content wiil be changed.
   (mew-current-set-window-config)
   (mew-window-configure 'draft)
   ;; main part
@@ -645,15 +711,19 @@ The message is assumed to be a valid MIME message."
    (let* ((file (mew-expand-folder fld msg))
 	  (work (concat file mew-queue-work-suffix))
 	  (info (concat file mew-queue-info-suffix))
-	  inf hdr case)
+	  inf hdr)
      (when (file-readable-p file)
        (rename-file file work 'override)
        (setq inf (mew-lisp-load info))
-       (setq hdr (mew-smtp-get-raw-header inf)) ;; fcc/dcc/bcc included
-       (setq case (mew-smtp-get-case inf))
-       (mew-queue-backup file)
-       (mew-tinfo-set-case case)
-       (mew-summary-edit-header-for-queue hdr)))
+       (cond
+	((mew-folder-queuep fld)
+	 (setq hdr (mew-smtp-get-raw-header inf))
+	 (mew-tinfo-set-case (mew-smtp-get-case inf)))
+	((mew-folder-postqp fld)
+	 (setq hdr (mew-nntp2-get-raw-header inf))
+	 (mew-tinfo-set-case (mew-nntp2-get-case inf))))
+       (mew-queue-backup file mew-queue-info-suffix)
+       (and hdr (mew-summary-edit-header-for-queue hdr))))
    (mew-draft-mode)
    (mew-draft-rehighlight-body)
    (when mew-encode-syntax
@@ -679,7 +749,7 @@ The message is assumed to be a valid MIME message."
        (mew-draft-prepare-attachments t)))
    (mew-draft-mode-name)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Edit again
 ;;;
@@ -721,10 +791,10 @@ mew-summary-reedit."
 	 ;; A new message is stored in the temporary directory
 	 (mew-summary-reedit-for-message nfld nmsg nil nil nil)
 	 (run-hooks 'mew-draft-mode-edit-again-hook)
-	 (if (file-exists-p nfile) (delete-file nfile))))
+	 (mew-delete-file nfile)))
      (mew-remove-buffer buf))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; SUb functions
 ;;;
@@ -847,7 +917,7 @@ mew-summary-reedit."
 	 (beg (mew-syntax-get-begin syntax))
 	 (end (mew-syntax-get-end syntax))
 	 (linebasep (or (mew-ct-textp ct) (mew-ct-linebasep ct))))
-    (if (and (stringp file) (string-match "^[-_+#.a-zA-Z0-9]+$" file))
+    (if (stringp file)
 	(setq file (expand-file-name file dir))
       (setq file (mew-random-filename dir 2 nil (mew-ct-suffix ct)))
       (mew-syntax-set-cdp syntax nil))
@@ -910,7 +980,7 @@ mew-summary-reedit."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2001 Mew developing team.
+;; Copyright (C) 1999-2002 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
