@@ -2,13 +2,12 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct 18, 1997
-;; Revised: Jul  8, 2001
 
 ;;; Code:
 
 (require 'mew)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Cursor line
 ;;;
@@ -37,7 +36,11 @@
     (set-marker overlay-arrow-position
 		(save-excursion (beginning-of-line) (point)))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defun mew-unhighlight-cursor-line ()
+  (if (overlayp (mew-sinfo-get-cursor-line))
+      (move-overlay (mew-sinfo-get-cursor-line) 1 1)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Marks
 ;;;
@@ -57,7 +60,7 @@
    (save-excursion (end-of-line) (point))
    '(face nil)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Header
 ;;;
@@ -83,7 +86,7 @@
 	   (while (not (eobp))
 	     (if (not (looking-at mew-keyval))
 		 (forward-line)
-	       (setq key (mew-match 1))
+	       (setq key (mew-match-string 1))
 	       (setq beg (match-beginning 0))
 	       (setq med (match-end 0))
 	       (forward-line)
@@ -107,7 +110,7 @@
     (widen)
     (mew-highlight-header-region (point-min) (mew-header-end))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Body
 ;;;
@@ -136,7 +139,7 @@
 	     (setq beg (match-beginning 0))
 	     (setq end (match-end 0))
 	     (if (match-beginning 1)
-		 (setq fancy-prefix (mew-match 1))
+		 (setq fancy-prefix (mew-match-string 1))
 	       (setq fancy-prefix nil))
 	     (when (and fancy-prefix
 			(< (string-width fancy-prefix)
@@ -164,7 +167,7 @@
 	     (put-text-property
 	      beg end 'mouse-face mew-highlight-url-mouse-face))))))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; X-Face:
 ;;;
@@ -184,31 +187,30 @@
       (require 'highlight-headers)) ;; due to the timing problem.
   ;; now this is in the "mail-lib" package.
   (defvar mew-use-highlight-x-face-function
-    (function
-     (lambda (beg end)
-       (when (and window-system
-		  (mew-which-exec mew-prog-uncompface)
-		  mew-use-highlight-x-face)
-	 (save-excursion
-	   (goto-char beg)
-	   (mew-elet
-	    (let ((regex1 "^X-Face: *\\(.*\\(\n[ \t].*\\)*\\)\n")
-		  (regex2 (concat "^\\(" mew-from: "\\).*"))
-		  overlay xface)
-	      (while (re-search-forward regex1 end t)
-		(setq overlay (mew-overlay-make (match-beginning 0)
-						(match-end 0)))
-		(overlay-put overlay 'invisible t)
-		(mew-plet
-		 (setq xface (highlight-headers-x-face-to-pixmap
-			      (match-beginning 1)
-			      (match-end 1))))
-		(save-excursion
-		  (goto-char beg)
-		  (when (re-search-forward regex2 end t)
-		    (setq overlay (mew-overlay-make
-				   (match-end 1) (match-end 1)))
-		    (set-extent-begin-glyph overlay xface))))))))))))
+    (lambda (beg end)
+      (when (and window-system
+		 (mew-which-exec mew-prog-uncompface)
+		 mew-use-highlight-x-face)
+	(save-excursion
+	  (goto-char beg)
+	  (mew-elet
+	   (let ((regex1 "^X-Face: *\\(.*\\(\n[ \t].*\\)*\\)\n")
+		 (regex2 (concat "^\\(" mew-from: "\\).*"))
+		 overlay xface)
+	     (while (re-search-forward regex1 end t)
+	       (setq overlay (mew-overlay-make (match-beginning 0)
+					       (match-end 0)))
+	       (overlay-put overlay 'invisible t)
+	       (mew-plet
+		(setq xface (highlight-headers-x-face-to-pixmap
+			     (match-beginning 1)
+			     (match-end 1))))
+	       (save-excursion
+		 (goto-char beg)
+		 (when (re-search-forward regex2 end t)
+		   (setq overlay (mew-overlay-make
+				  (match-end 1) (match-end 1)))
+		   (set-extent-begin-glyph overlay xface)))))))))))
  (t
   (defvar mew-use-highlight-x-face-function nil
     "*On Text Emacs, this function is called if mew-use-highlight-x-face
@@ -219,7 +221,7 @@ is *non-nil*. This is a temporary solution.")
 	   (mew-which-el "mew-xface-mule"))
       (require 'mew-xface-mule))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Cooking
 ;;;
@@ -236,7 +238,9 @@ is *non-nil*. This is a temporary solution.")
 ;; See also mew-scan-insert-line
 (defun mew-summary-cook-region (beg end &optional interrupt)
   (when (and (mew-summary-or-virtual-p) mew-summary-buffer-raw)
-    (let ((inhibit-point-motion-hooks t) ret regex mark face start med)
+    (let ((inhibit-point-motion-hooks t)
+	  (regex mew-regex-msg-mark)
+	  ret mark face start med)
       (catch 'loop
 	(save-excursion
 	  (mew-elet
@@ -248,7 +252,7 @@ is *non-nil*. This is a temporary solution.")
 		    (not (mew-summary-message-number)))
 	       (forward-line))
 	   (setq start (point))
-	   (while (and (< (point) end) ;; we can't trust end
+	   (while (and (< (point) end) ;; we cannot trust end
 		       (search-forward "\r" end t))
 	     (if (and interrupt (input-pending-p))
 		 (throw 'loop (setq ret t)))
@@ -268,12 +272,11 @@ is *non-nil*. This is a temporary solution.")
 	     (setq start (point)))
 	   (when mew-use-highlight-mark
 	     (goto-char beg)
-	     (setq regex (concat mew-regex-msg "\\([^ ]\\)"))
-	     (while (and (< (point) end) ;; we can't trust end
+	     (while (and (< (point) end) ;; we cannot trust end
 			 (re-search-forward regex end t))
 	       (if (and interrupt (input-pending-p))
 		   (throw 'loop (setq ret t)))
-	       (setq mark (string-to-char (mew-match 2)))
+	       (setq mark (string-to-char (mew-match-string 2)))
 	       (setq face (mew-highlight-mark-get-face mark))
 	       (beginning-of-line)
 	       (setq start (point))
@@ -310,7 +313,7 @@ is *non-nil*. This is a temporary solution.")
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2001 Mew developing team.
+;; Copyright (C) 1997-2002 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
