@@ -17,7 +17,7 @@
 (defun mew-mime-start-process (program options file)
   (let ((process-connection-type mew-connection-type1) pro)
     (message "Starting %s..." program)
-    (setq pro (apply (function mew-start-process-disp)
+    (setq pro (apply 'mew-start-process-disp
 		     (format "*mew %s*" program)
 		     nil
 		     program
@@ -30,12 +30,20 @@
 (defun mew-mime-start-process-sentinel (process event)
   (let* ((al (assoc process mew-process-file-alist))
 	 (file (cdr al)))
-    (if (and mew-delete-temp-file file) (delete-file file))
+    ;; A launcher program may be executed.
+    ;; The launcher program executes an application according to
+    ;; file's suffix or something.
+    ;; The time when the launcher is finished is not the time when
+    ;; the application is finished. So, we can't delete the temporary
+    ;; file here. Hoping that the file will be deleted when Mew is 
+    ;; finished.
+    (if mew-delete-temp-file (mew-delete-file file))
     (setq mew-process-file-alist (delq al mew-process-file-alist))))
 
 (defun mew-mime-call-process (program options file)
   (message "Calling %s..." program)
-  (apply (function call-process) program file nil nil options)
+  (apply 'call-process program file nil nil options)
+  (mew-delete-file file)
   (message "Calling %s...done" program)
   t) ;; to next part
 
@@ -125,10 +133,10 @@
   (when (> end begin)
     (save-excursion
       (mew-elet
-       ;; Need to inherit properties in a cache.
-       ;; So, we use buffer-substring, not mew-buffer-substring.
-       (insert
-	(save-excursion (set-buffer cache) (buffer-substring begin end)))
+       ;; We need to keep properties (e.g. citation color)
+       ;; in a message cache.
+       ;; This must be "insert-buffer-substring".
+       (insert-buffer-substring cache begin end)
        (when mew-insert-final-newline
 	 (save-excursion
 	   (goto-char (point-max))
@@ -144,11 +152,14 @@
     (set-buffer (mew-buffer-message))
     (mew-elet
      (mew-erase-buffer)
-     (insert
-      (save-excursion (set-buffer cache) (buffer-substring begin end)))
-     ;; If the "charset" parameter exists, this text is already decoded.
-     (unless (or mew-decode-broken
-		 (mew-charset-to-cs (mew-syntax-get-param params "charset")))
+     ;; We need to keep properties (e.g. citation color)
+     ;; in a message cache.
+     ;; This must be "insert-buffer-substring".
+     (insert-buffer-substring cache begin end)
+     (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
+		  (or (mew-cache-dinfo-get-decode-broken cache)
+		      (mew-charset-to-cs
+		       (mew-syntax-get-param params "charset"))))
        (mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv))
      (when mew-insert-final-newline
        (save-excursion
@@ -171,7 +182,9 @@
     (save-excursion
       (let ((start (point)))
 	(mew-elet
-	 (mew-insert-buffer-substring cache begin end)
+	 ;; We need to keep composite properties of charset.
+	 ;; This must be "insert-buffer-substring".
+	 (insert-buffer-substring cache begin end)
 	 ;; Highlight
 	 (when mew-use-text/enriched
 	   (condition-case nil
@@ -195,9 +208,9 @@
       (save-excursion
 	(mew-elet
 	 (let ((start (point)))
-	   (insert
-	    (save-excursion
-	      (set-buffer cache) (mew-buffer-substring begin end)))
+	   ;; We need to keep composite properties of charset.
+	   ;; This must be "insert-buffer-substring".
+	   (insert-buffer-substring cache begin end)
 	   (funcall mew-prog-text/html start (point-max)))))
     (mew-elet
      (insert " #     # ####### #     # #\n"
@@ -224,11 +237,13 @@
     (save-excursion
       (mew-elet
        (mew-erase-buffer)
-       (insert
-	(save-excursion (set-buffer cache) (buffer-substring begin end)))
-       ;; If the "charset" parameter exists, this text is already decoded.
-       (unless (or mew-decode-broken
-		   (mew-charset-to-cs (mew-syntax-get-param params "charset")))
+       ;; We need to keep composite properties of charset.
+       ;; This must be "insert-buffer-substring".
+       (insert-buffer-substring cache begin end)
+       (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
+		    (or (mew-cache-dinfo-get-decode-broken cache)
+			(mew-charset-to-cs
+			 (mew-syntax-get-param params "charset"))))
 	 (mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv))
        (funcall mew-prog-text/html-ext (point-min) (point-max))
        (mew-summary-display-postscript 'nohook)))
@@ -254,7 +269,7 @@
 	  (mew-frwlet
 	   mew-cs-dummy wcs
 	   (write-region begin end file nil 'no-msg)
-	   (apply (function mew-start-process-disp)
+	   (apply 'mew-start-process-disp
 		  mew-prog-text/html-ext nil mew-prog-text/html-ext arg))
 	  (message "Displaying Text/Html...done")))))))
 
@@ -274,10 +289,10 @@
 		     ";[ \t]*charset=\"?\\([^\"]+\\)\"?"
 		     "[ \t]+http-equiv=\"?content-type\"?[ \t]*/?>")
 	     (min end (+ begin 1024)) t))
-	(mew-charset-to-cs (mew-match 1)))
+	(mew-charset-to-cs (mew-match-string 1)))
        (cs cs)
        (t  ;; charset eq "us-ascii", cs is nil.
-	(if mew-decode-broken
+	(if mew-decode-broken ;; xxx
 	    (mew-charset-to-cs (mew-charset-guess-region begin end))
 	  ;; this text is no decoded.
 	  mew-cs-text-for-write))))))
@@ -293,9 +308,9 @@
       (save-excursion
 	(mew-elet
 	 (let ((start (point)))
-	   (insert
-	    (save-excursion
-	      (set-buffer cache) (mew-buffer-substring begin end)))
+	   ;; We need to keep composite properties of charset.
+	   ;; This must be "insert-buffer-substring".
+	   (insert-buffer-substring cache begin end)
 	   (funcall mew-prog-text/xml start (point-max)))))
     (mew-elet
      (insert " #     # #     # #\n"
@@ -322,11 +337,13 @@
     (save-excursion
       (mew-elet
        (mew-erase-buffer)
-       (insert
-	(save-excursion (set-buffer cache) (buffer-substring begin end)))
-       ;; If the "charset" parameter exists, this text is already decoded.
-       (unless (or mew-decode-broken
-		   (mew-charset-to-cs (mew-syntax-get-param params "charset")))
+       ;; We need to keep composite properties of charset.
+       ;; This must be "insert-buffer-substring".
+       (insert-buffer-substring cache begin end)
+       (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
+		    (or (mew-cache-dinfo-get-decode-broken cache)
+			(mew-charset-to-cs
+			 (mew-syntax-get-param params "charset"))))
 	 (mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv))
        (funcall mew-prog-text/xml-ext (point-min) (point-max))
        (mew-summary-display-postscript 'nohook)))
@@ -352,7 +369,7 @@
 	  (mew-frwlet
 	   mew-cs-dummy wcs
 	   (write-region begin end file nil 'no-msg)
-	   (apply (function mew-start-process-disp)
+	   (apply 'mew-start-process-disp
 		  mew-prog-text/xml-ext nil mew-prog-text/xml-ext arg))
 	  (message "Displaying Text/Xml...done")))))))
 
@@ -365,7 +382,7 @@
        ((re-search-forward
 	 "<\\?xml.*encoding=['\"]\\([^'\"]+\\)['\"] *\\?>"
 	 (min end (+ begin 1024)) t)
-	(mew-charset-to-cs (mew-match 1)))
+	(mew-charset-to-cs (mew-match-string 1)))
        (cs cs)
        (t  ;; charset eq "us-ascii", cs is nil.
 	(if mew-decode-broken
@@ -385,9 +402,9 @@
       (save-excursion
 	(mew-elet
 	 (let ((start (point)))
-	   (insert
-	    (save-excursion
-	      (set-buffer cache) (mew-buffer-substring begin end)))
+	   ;; We need to keep composite properties of charset.
+	   ;; This must be "insert-buffer-substring".
+	   (insert-buffer-substring cache begin end)
 	   (funcall mew-prog-application/xml start (point-max)))))
     (mew-elet
      (insert " #     # #     # #\n"
@@ -414,8 +431,9 @@
     (save-excursion
       (mew-elet
        (mew-erase-buffer)
-       (insert
-	(save-excursion (set-buffer cache) (buffer-substring begin end)))
+       ;; We need to keep composite properties of charset.
+       ;; This must be "insert-buffer-substring".
+       (insert-buffer-substring cache begin end)
        (funcall mew-prog-application/xml-ext (point-min) (point-max))
        (mew-summary-display-postscript 'nohook)))
     (message "Displaying XML...done"))
@@ -440,7 +458,7 @@
 	  (mew-frwlet
 	   mew-cs-dummy wcs
 	   (write-region begin end file nil 'no-msg)
-	   (apply (function mew-start-process-disp)
+	   (apply 'mew-start-process-disp
 		  mew-prog-application/xml-ext nil
 		  mew-prog-application/xml-ext arg))
 	  (message "Displaying Application/Xml...done")))))))
@@ -482,11 +500,23 @@
 ;;; Message/Rfc822
 ;;;
 
+(defun mew-mime-text/rfc822-headers (cache begin end &optional params)
+  ;; called in Message buffer
+  (when (> end begin)
+    (save-excursion
+      (mew-elet
+       ;; We need to keep properties of a header.
+       ;; This must be "insert-buffer-substring".
+       (insert-buffer-substring cache begin end)
+       (mew-header-arrange (point-min) (point-max))))))
+
 (defun mew-mime-message/rfc822 (cache part)
+  ;; called in Message buffer
   (mew-elet
    (let* ((hbeg (mew-syntax-get-begin part))
 	  (hend (mew-syntax-get-end   part))
 	  (body (mew-syntax-get-part part)))
+     ;; We need to keep properties of a header.
      ;; This must be "insert-buffer-substring".
      (insert-buffer-substring cache hbeg hend)
      (mew-header-arrange (point-min) (point-max))
@@ -522,9 +552,7 @@
    (when params
      (mew-insert
       "Parameters:\t%s\n"
-      (mapconcat
-       (function (lambda (x) (concat (nth 0 x) "=" (nth 1 x))))
-       params ", ")))
+      (mapconcat (lambda (x) (concat (nth 0 x) "=" (nth 1 x))) params ", ")))
    (mew-insert "Size:\t\t%d bytes\n" (mew-region-bytes begin end cache))
    (mew-insert "Filename:\t%s\n" fl)
    (when func (mew-insert "Function:\t%s\n" func))
@@ -612,8 +640,8 @@
 	(mew-frwlet
 	 'utf-8 mew-cs-dummy
 	 (call-process prog nil (current-buffer) nil file1)))
-      (if (file-exists-p file1) (delete-file file1))
-      (if (file-exists-p file2) (delete-file file2))
+      (mew-delete-file file1)
+      (mew-delete-file file2)
       (save-excursion
 	(w3m-region (point-min) (point-max)
 		    (w3m-expand-file-name-as-url (file-name-directory file2))))
