@@ -45,20 +45,64 @@
 (defvar mew-cs-m17n (if (mew-coding-system-p 'ctext-unix) 'ctext-unix 'ctext))
 
 (defvar mew-cs-eol "\n")
+(defun mew-eol-fix-for-read () ())
+(defun mew-eol-fix-for-write () ())
+
+;; Latin 0 -- Nickname of Latin 9
+;; Latin 1 -- ISO-8859-1   West European
+;; Latin 2 -- ISO-8859-2   East European
+;; Latin 3 -- ISO-8859-3   South European
+;; Latin 4 -- ISO-8859-4   North European
+;; Latin 5 -- ISO-8859-9   Turkish
+;; Latin 6 -- ISO-8859-10  Nordic
+;; Latin 7 -- ISO-8859-13  Baltic Rim
+;; Latin 8 -- ISO-8859-14  Celtic
+;; Latin 9 -- ISO-5589-15  New West European
+
+;; ISO-8859-1  -- Latin 1  West European
+;; ISO-8859-2  -- Latin 2  East European
+;; ISO-8859-3  -- Latin 3  South European
+;; ISO-8859-4  -- Latin 4  North European
+;; ISO-8859-5  --          Cyrillic
+;; ISO-8859-6  --          Arabic
+;; ISO-8859-7  --          Greek
+;; ISO-8859-8  --          Hebrew
+;; ISO-8859-9  -- Latin 5  Turkish
+;; ISO-8859-10 -- Latin 6  Nordic
+;; ISO-8859-11 --          Thai
+;; ISO-8859-12 --
+;; ISO-8859-13 -- Latin 7  Baltic Rim
+;; ISO-8859-14 -- Latin 8  Celtic
+;; ISO-8859-15 -- Latin 9  New West European  (or Latin 0)
 
 (defvar mew-cs-database-for-encoding
-  '(((ascii)                    nil        "7bit"             "7bit")
-    ((ascii latin-iso8859-1)    iso-8859-1 "quoted-printable" "Q")
-    ((ascii latin-iso8859-2)    iso-8859-2 "quoted-printable" "Q")
-    ((ascii latin-iso8859-3)    iso-8859-3 "quoted-printable" "Q")
-    ((ascii latin-iso8859-4)    iso-8859-4 "quoted-printable" "Q")
-    ((ascii cyrillic-iso8859-5) koi8-r     "quoted-printable" "Q")
-    ((ascii arabic-iso8859-6)   iso-8859-6 "base64"           "B")
-    ((ascii greek-iso8859-7)    iso-8859-7 "base64"           "B")
-    ((ascii hebrew-iso8859-8)   iso-8859-8 "base64"           "B")
-    ((ascii latin-iso8859-9)    iso-8859-9 "quoted-printable" "B")
+  '(((ascii)                    nil         "7bit"             "7bit")
+    ;; West European (Latin 1)
+    ((ascii latin-iso8859-1)    iso-8859-1  "quoted-printable" "Q")
+    ;; East European (Latin 2)
+    ((ascii latin-iso8859-2)    iso-8859-2  "quoted-printable" "Q")
+    ;; South European (Latin 3)
+    ((ascii latin-iso8859-3)    iso-8859-3  "quoted-printable" "Q")
+    ;; North European (Latin 4)
+    ((ascii latin-iso8859-4)    iso-8859-4  "quoted-printable" "Q")
+    ;; Cyrillic
+    ((ascii cyrillic-iso8859-5) koi8-r      "quoted-printable" "Q")
+    ;; Arabic
+    ((ascii arabic-iso8859-6)   iso-8859-6  "base64"           "B")
+    ;; Greek
+    ((ascii greek-iso8859-7)    iso-8859-7  "base64"           "B")
+    ;; Hebrew
+    ((ascii hebrew-iso8859-8)   iso-8859-8  "base64"           "B")
+    ;; Turkish (Latin 5)
+    ((ascii latin-iso8859-9)    iso-8859-9  "quoted-printable" "Q")
+    ;; Celtic (Latin 8)
+    ((ascii latin-iso8859-14)   iso-8859-14 "quoted-printable" "Q") ;; xxx
+    ;; New West European (Latin 9 or Latin 0)
+    ((ascii latin-iso8859-15)   iso-8859-15 "quoted-printable" "Q")
     ((ascii thai-tis620 composition) ;; composition for Emacs 20
-                                tis620     "base64"           "B")
+                                tis-620     "base64"           "B")
+    ((ascii thai-xtis) ;; thai-xtis for XEmacs
+                                tis-620     "base64"           "B")
     ((ascii latin-jisx0201 japanese-jisx0208 japanese-jisx0208-1978)
                                iso-2022-jp "7bit"             "B")
     ((ascii korean-ksc5601)     euc-kr     "8bit"             "B")
@@ -90,6 +134,7 @@
     ("iso-8859-7"      . iso-8859-7)
     ("iso-8859-8"      . iso-8859-8)
     ("iso-8859-9"      . iso-8859-9)
+    ("iso-8859-15"     . iso-8859-15)
     ("iso-2022-cn"     . iso-2022-cn)
     ("iso-2022-cn-ext" . iso-2022-cn-ext)
     ("gb2312"          . cn-gb-2312) ;; should be before cn-gb
@@ -106,7 +151,16 @@
     ("euc-jp"          . euc-japan)
     ("shift_jis"       . shift_jis)
     ("koi8-r"          . koi8-r)
-    ("tis-620"         . tis620)
+    ("windows-1250"    . cp1250)
+    ("windows-1251"    . cp1251)
+    ("windows-1252"    . cp1252)
+    ("windows-1253"    . cp1253)
+    ("windows-1254"    . cp1254)
+    ("windows-1255"    . cp1255)
+    ("windows-1256"    . cp1256)
+    ("windows-1257"    . cp1257)
+    ("windows-1258"    . cp1258)
+    ("tis-620"         . tis-620)
     ("iso-2022-int-1"  . iso-2022-int-1)
     ("utf-7"           . utf-7)
     ("utf-8"           . utf-8)))
@@ -150,7 +204,7 @@
 	  (insert str)
 	  (or (mew-multibyte-string-p str) (mew-set-buffer-multibyte t))
 	  (decode-coding-region (point-min) (point-max) cs)
-	  (setq ret (buffer-substring (point-min) (point-max))))
+	  (setq ret (buffer-substring (point-min) (point-max)))) ;; xxx
 	(mew-remove-buffer buf)
 	ret))))
 
@@ -178,24 +232,27 @@
 	 (coding-system-for-write ,write))
      ,@body))
 
-(defmacro mew-pioalet (read write arg &rest body)
-  `(let ((coding-system-for-read  ,read)
-	 (coding-system-for-write ,write)
-	 (file-name-coding-system ,arg))
-     ,@body))
-
 (defmacro mew-flet (&rest body)
   `(let ((coding-system-for-read  'binary)
 	 (coding-system-for-write 'binary)
+	 (format-alist nil)
+	 (auto-image-file-mode nil)
 	 (jka-compr-inhibit t))
      ,@body))
 
 (defmacro mew-frwlet (read write &rest body)
   `(let ((coding-system-for-read  ,read)
 	 (coding-system-for-write ,write)
+	 (format-alist nil)
+	 (auto-image-file-mode nil)
 	 (jka-compr-inhibit t))
      ,@body))
 
+(defmacro mew-alet (&rest body)
+  `(let ((default-file-name-coding-system nil)
+	 (file-name-coding-system nil))
+     ,@body))
+
 ;;
 ;;
 ;;
@@ -293,7 +350,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2001 Mew developing team.
+;; Copyright (C) 1997-2002 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
