@@ -96,13 +96,13 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 (defconst mew-pgp-msg-signature
   '("\n\\(.*\\) signature from user \\(.*\\)\\."
     "\n\\(.*\\) signature made"
-    "\\([0-9A-Za-z]*\\) signature from user \\(.*\\)\\."
+    "\\([a-zA-Z0-9]*\\) signature from user \\(.*\\)\\."
     " \\(.*\\) signature from \"\\(.*\\)\""))
 
 (defconst mew-pgp-msg-key-id
   '("Key ID \\(\\S +\\) not found"
-    ": \\(0x[0-9A-Za-z]+\\)"
-    ": \\(0x[0-9A-Za-z]+\\)"
+    ": \\(0x[a-zA-Z0-9]+\\)"
+    ": \\(0x[a-zA-Z0-9]+\\)"
     "xxx"))
 
 (defconst mew-pgp-msg-bad-pass
@@ -241,7 +241,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
       (goto-char (point-max))
       (insert (format "\n<%s>\n%s\n" label string)))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; PGP version check
 ;;;
@@ -270,7 +270,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
       (mew-remove-buffer buf)))
    (t (setq mew-pgp-ver nil))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; PGP selection
 ;;;
@@ -280,9 +280,8 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
   (interactive)
   (setq mew-prog-pgp
 	(completing-read
-	 "PGP name : "
-	 (mapcar (function (lambda (x) (cons x x)))
-		 (list mew-prog-pgp2 mew-prog-pgp5 mew-prog-gpg))
+	 "PGP name: "
+	 (mapcar 'list (list mew-prog-pgp2 mew-prog-pgp5 mew-prog-gpg))
 	 nil t))
   (mew-pgp-setup))
 
@@ -298,7 +297,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     (setq prompt (format prompt msg))
     (mew-input-passwd prompt msg)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; PGP verifying
 ;;;
@@ -333,17 +332,17 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 		;; from the outside of the cipher.
 		))))
       ;; Signature result is found.
-      (setq ret (concat (mew-match 1) " PGP sign "))
+      (setq ret (concat (mew-match-string 1) " PGP sign "))
       (goto-char (point-max))
-      (if (and (boundp 'mew-inherit-decode-signer) mew-inherit-decode-signer
-	       (re-search-backward (concat (concat (mew-pgp-get mew-pgp-verify-addr) ".*") mew-inherit-decode-signer) nil t))
+      (if (and mew-inherit-decode-signer
+	       (re-search-backward (concat (mew-pgp-get mew-pgp-verify-addr) ".*" mew-inherit-decode-signer) nil t))
 	  (progn
 	    (beginning-of-line)
 	    (looking-at (concat (mew-pgp-get mew-pgp-verify-addr) "\\(.*\\)"))
-	    (setq ret (concat ret (mew-match 2))))
+	    (setq ret (concat ret (mew-match-string 2))))
 	(goto-char (point-max))
 	(re-search-backward (concat (mew-pgp-get mew-pgp-verify-addr) "\\(.*\\)") nil t)
-	(setq ret (concat ret (mew-match 2))))
+	(setq ret (concat ret (mew-match-string 2))))
       ;; xxx
       (goto-char (point-min))
       (if (search-forward "not certified with enough" nil t)
@@ -371,12 +370,12 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
       (set-buffer buf)
       (mew-erase-buffer)
       (if ioption
-	  (apply (function mew-call-process-lang)
+	  (apply 'mew-call-process-lang
 		 (mew-pgp-get mew-prog-pgpv)
 		 nil t nil 
 		 (append (mew-pgp-get mew-prog-pgpv-arg)
 			 (list ioption file1 file2)))
-	(apply (function mew-call-process-lang)
+	(apply 'mew-call-process-lang
 	       (mew-pgp-get mew-prog-pgpv)
 	       nil t nil 
 	       (append (mew-pgp-get mew-prog-pgpv-arg)
@@ -394,7 +393,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     (save-excursion
       (set-buffer buf)
       (mew-erase-buffer)
-      (apply (function mew-call-process-lang)
+      (apply 'mew-call-process-lang
 	     (mew-pgp-get mew-prog-pgpv)
 	     nil t nil 
 	     (append (mew-pgp-get mew-prog-old-pgpv-arg)
@@ -404,7 +403,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     (message "PGP verifying...done")
     (list file1 verify)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; PGP encrypting
 ;;;
@@ -442,14 +441,13 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	      ;; nreverse later, take care.
 	      (setq decrypters (cdr decrypters)))
 	    (setq decrypters (nreverse decs))
-	    (apply
-	     (function mew-call-process-lang) 
-	     (mew-pgp-get mew-prog-pgpe)
-	     nil t nil
-	     (append (mew-pgp-get mew-prog-pgpe-arg)
-		     decrypters
-		     (list (mew-pgp-get mew-prog-arg-output) file3 file1))))
-	(apply (function mew-call-process-lang) 
+	    (apply 'mew-call-process-lang
+		   (mew-pgp-get mew-prog-pgpe)
+		   nil t nil
+		   (append (mew-pgp-get mew-prog-pgpe-arg)
+			   decrypters
+			   (list (mew-pgp-get mew-prog-arg-output) file3 file1))))
+	(apply 'mew-call-process-lang
 	       (mew-pgp-get mew-prog-pgpe)
 	       nil t nil
 	       (mew-pgp-get mew-prog-arg-output) file3 file1 
@@ -459,7 +457,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
       (message "PGP encrypting...done")
       (list file2 "7bit" file3 "7bit" check)))) ;; both ctes are 7bit
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; PGP decrypting
 ;;;
@@ -476,13 +474,12 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	file3 process verify)
     (setq file3 (mew-make-temp-name))
     (setq process
-	  (apply
-	   (function mew-start-process-lang)
-	   "PGP decrypt"
-	   (current-buffer)
-	   (mew-pgp-get mew-prog-pgpd)
-	   (append (mew-pgp-get mew-prog-pgpd-arg)
-		   (list (mew-pgp-get mew-prog-arg-output) file3 file2))))
+	  (apply 'mew-start-process-lang
+		 "PGP decrypt"
+		 (current-buffer)
+		 (mew-pgp-get mew-prog-pgpd)
+		 (append (mew-pgp-get mew-prog-pgpd-arg)
+			 (list (mew-pgp-get mew-prog-arg-output) file3 file2))))
     (mew-set-process-cs process mew-cs-autoconv mew-cs-dummy)
     (set-process-filter process 'mew-pgp-process-filter1)
     (set-process-sentinel process 'mew-pgp-process-sentinel)
@@ -505,7 +502,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	  (setq mew-pgp-decrypt-msg mew-pgp-result-dec-fail)))
     (list file3 mew-pgp-decrypt-msg)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; PGP signing
 ;;;
@@ -522,7 +519,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	(insert mew-pgp-string)
 	(goto-char (point-min))
 	(if (re-search-forward "SIG_CREATED [A-Z] [0-9]+ \\([0-9]+\\)" nil t)
-	   (setq alg (cdr (assoc (mew-match 1) mew-pgp-hash-alist)))))
+	   (setq alg (cdr (assoc (mew-match-string 1) mew-pgp-hash-alist)))))
       (mew-remove-buffer buf)
       (or alg micalg))))
 
@@ -537,14 +534,13 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     (setq file2 (concat (mew-make-temp-name) mew-pgp-ascii-suffix))
     ;; not perfectly unique but OK
     (setq process
-	  (apply
-	   (function mew-start-process-lang)
-	   "PGP sign"
-	   (current-buffer)
-	   (mew-pgp-get mew-prog-pgps)
-	   (append (mew-pgp-get mew-prog-pgps-arg)
-		   (list (mew-pgp-get mew-prog-arg-luserid) mew-inherit-encode-signer
-			 (mew-pgp-get mew-prog-arg-output) file2 file1))))
+	  (apply 'mew-start-process-lang
+		 "PGP sign"
+		 (current-buffer)
+		 (mew-pgp-get mew-prog-pgps)
+		 (append (mew-pgp-get mew-prog-pgps-arg)
+			 (list (mew-pgp-get mew-prog-arg-luserid) mew-inherit-encode-signer
+			       (mew-pgp-get mew-prog-arg-output) file2 file1))))
     (mew-set-process-cs process mew-cs-autoconv mew-cs-dummy)
     (set-process-filter process 'mew-pgp-process-filter1)
     (set-process-sentinel process 'mew-pgp-process-sentinel)
@@ -554,7 +550,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	(mew-passwd-set-passwd (mew-pgp-passtag) nil))
     (list file2 nil (mew-pgp-get-micalg) mew-pgp-sign-msg))) ;; return
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; PGP process functions
 ;;;
@@ -701,7 +697,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     (process-send-string process "\n")
     (set-process-filter process 'mew-pgp-process-filter3))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; shortcut methods
 ;;;
@@ -730,10 +726,10 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
   (if (null mew-pgp-ver)
       (message "%s does not exist" mew-prog-pgp)
     (if (and ask-singer (string-match "signature" (symbol-name type)))
-	(mew-draft-make-message type (car (mew-input-address "Who's key? : ")))
+	(mew-draft-make-message type (car (mew-input-address "Who's key?: ")))
       (mew-draft-make-message type))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; key distribution
 ;;;
@@ -798,14 +794,14 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	      (mew-frwlet
 	       mew-cs-text-for-read mew-cs-dummy
 	       (insert-file-contents file1))
-	      (delete-file file1))
+	      (mew-delete-file file1))
 	    (if mew-mule-p
 		(mew-cs-decode-region (point) (point-max) mew-cs-autoconv))
 	    (mew-message-set-end-of)
 	    (set-window-start win start)
 	    (set-buffer-modified-p nil))))))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Key distribution
 ;;;
@@ -834,7 +830,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	(save-excursion
 	  (set-buffer buf)
 	  (mew-erase-buffer)
-	  (apply (function mew-call-process-lang)
+	  (apply 'mew-call-process-lang
 		 (mew-pgp-get mew-prog-pgpk)
 		 nil t nil
 		 (append
@@ -896,7 +892,7 @@ public keyring."
 	(set-buffer (mew-buffer-message))
 	(mew-elet
 	 (message "Adding PGP keys...")
-	 (apply (function mew-call-process-lang)
+	 (apply 'mew-call-process-lang
 		(mew-pgp-get mew-prog-pgpk)
 		nil t nil 
 		(append (mew-pgp-get mew-prog-pgpk-add-arg)
@@ -922,10 +918,9 @@ public keyring."
 		 "**************** IMPORTANT NOTE ****************\n")
 	 ;; (let ((inhibit-quit t)) (read-char-exclusive)) ;; xxx
 	 ))
-      (if (file-exists-p mew-pgp-tmp-file)
-	  (delete-file mew-pgp-tmp-file)))))
+      (mew-delete-file mew-pgp-tmp-file))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; key fetch
 ;;;
@@ -980,7 +975,7 @@ public keyring."
     (mew-piolet
      mew-cs-autoconv mew-cs-autoconv
      (setq mew-pgp-fetch-process
-	   (apply (function start-process)
+	   (apply 'start-process
 		  "Mew PGP fetch" buf
 		  mew-prog-pgpkey
 		  (append mew-prog-pgpkey-arg (list uri)))))
