@@ -20,75 +20,86 @@ execute 'mew-summary-pick-with-external'. Otherwise,
   (interactive "P")
   (mew-summary-or-thread
    (let ((folder (mew-summary-folder-name))
-	 (pattern (if arg (mew-input-grep-pattern) (mew-input-pick-pattern)))
 	 (preline 0)
-	 msgs msgsback threadmsgs total linenum n)
-     (mew-sinfo-set-find-key nil) ;; force to ask a user
-     (message "Picking messages in %s..." folder)
-     (if arg
-	 (setq msgs (mew-summary-pick-with-external folder pattern))
-       (setq msgs (mew-summary-pick-with-mewls folder pattern)))
-     (setq msgsback msgs)
-     (message "Picking messages in %s...done" folder)
-     (if (null msgs)
-	 (message "No message to be marked")
-       (setq n (length msgs))
-       (if (= n 1)
- 	   (message "Marking 1 message...")
- 	 (message "Marking %d messages..." n))
+	 pattern msgs msgsback threadmsgs total linenum n)
+     (unless folder
+       ;; probably on a thread separator
        (save-excursion
-	 ;; Thread folder
-	 (when (mew-thread-p)
-	   (setq threadmsgs (mew-summary-thread-get-msglst
-			     (mew-vinfo-get-top) 'separator))
-	   (setq total (length threadmsgs))
-	   (goto-char (point-min))
-	   (while (and msgs (not (eobp)))
-	     (setq linenum (member (car msgs) threadmsgs))
-	     (if (null linenum)
-		 (setq msgsback (delete (car msgs) msgsback))
-	       (setq linenum (- total (length linenum)))
-	       (forward-line (- linenum preline))
-	       (setq preline linenum)
-	       (mew-summary-mark-as mew-mark-review))
-	     (setq msgs (cdr msgs)))
-	   (set-buffer-modified-p nil))
-	 ;; Physical folder
-	 (when (get-buffer folder)
-	   (set-buffer folder)
-	   (save-excursion
+	 (goto-char (point-min))
+	 (setq folder (mew-summary-folder-name))))
+     (if (null folder)
+	 (message "No message to be picked")
+       (setq pattern (if arg (mew-input-grep-pattern)
+		       (mew-input-pick-pattern)))
+       (mew-sinfo-set-find-key nil);; force to ask a user
+       (message "Picking messages in %s..." folder)
+       (if arg
+	   (setq msgs (mew-summary-pick-with-external folder pattern))
+	 (setq msgs (mew-summary-pick-with-mewls folder pattern)))
+       (setq msgsback msgs)
+       (message "Picking messages in %s...done" folder)
+       (if (null msgs)
+	   (message "No message to be marked")
+	 (setq n (length msgs))
+	 (if (= n 1)
+	     (message "Marking 1 message...")
+	   (message "Marking %d messages..." n))
+	 (save-excursion
+	   ;; Thread folder
+	   (when (mew-thread-p)
+	     (setq threadmsgs (mew-summary-thread-get-msglst
+			       (mew-vinfo-get-top) 'separator))
+	     (setq total (length threadmsgs))
 	     (goto-char (point-min))
-	     (setq msgs msgsback)
 	     (while (and msgs (not (eobp)))
-	       (when (re-search-forward (mew-regex-jmp-msg (car msgs)) nil t)
-		 (mew-summary-mark-as mew-mark-review)
-		 (forward-line))
+	       (setq linenum (member (car msgs) threadmsgs))
+	       (if (null linenum)
+		   (setq msgsback (delete (car msgs) msgsback))
+		 (setq linenum (- total (length linenum)))
+		 (forward-line (- linenum preline))
+		 (setq preline linenum)
+		 (mew-summary-mark-as mew-mark-review))
 	       (setq msgs (cdr msgs)))
-	     (set-buffer-modified-p nil))))
-       (if (= n 1)
- 	   (message "Marking 1 message...done")
- 	 (message "Marking %d messages...done" n))))))
+	     (set-buffer-modified-p nil))
+	   ;; Physical folder
+	   (when (get-buffer folder)
+	     (set-buffer folder)
+	     (save-excursion
+	       (goto-char (point-min))
+	       (setq msgs msgsback)
+	       (while (and msgs (not (eobp)))
+		 (when (re-search-forward (mew-regex-jmp-msg (car msgs)) nil t)
+		   (mew-summary-mark-as mew-mark-review)
+		   (forward-line))
+		 (setq msgs (cdr msgs)))
+	       (set-buffer-modified-p nil))))
+	 (if (= n 1)
+	     (message "Marking 1 message...done")
+	   (message "Marking %d messages...done" n)))))))
 
 (defun mew-summary-pick-with-mewls (folder pattern)
   "A function to pick messages matching PATTERN with 'mewls'"
-  (let ((buf (generate-new-buffer mew-buffer-prefix))
-	msgs)
-    (save-excursion
-      (set-buffer buf)
-      (mew-erase-buffer)
-      (mew-set-buffer-multibyte t)
-      (mew-piolet
-       mew-cs-text-for-read mew-cs-text-for-write
-       (call-process mew-prog-mewls nil t nil
-		     "-b" mew-mail-path "-c" mew-news-path
-		     "-p" pattern "-s" folder))
-      (goto-char (point-min))
-      (while (not (eobp))
-	(if (looking-at mew-regex-message-files)
-	    (setq msgs (cons (mew-match 0) msgs)))
-	(forward-line)))
-    (mew-remove-buffer buf)
-    (nreverse msgs)))
+  (mew-summary-with-mewls
+   (let ((buf (generate-new-buffer mew-buffer-prefix))
+	 (fld (mew-expand-folder2 folder))
+	 msgs)
+     (save-excursion
+       (set-buffer buf)
+       (mew-erase-buffer)
+       (mew-set-buffer-multibyte t)
+       (mew-piolet
+	mew-cs-text-for-read mew-cs-text-for-write
+	(call-process mew-prog-mewls nil t nil
+		      "-b" mew-mail-path
+		      "-l" (int-to-string mew-scan-max-field-length)
+		      "-p" pattern "-s" fld))
+       (goto-char (point-min))
+       (while (not (eobp))
+	 (if (looking-at mew-regex-message-files)
+	     (setq msgs (cons (mew-match-string 0) msgs)))
+	 (forward-line)))
+     (mew-remove-buffer buf)
+     (nreverse msgs))))
 
 (defvar mew-summary-pick-with-external-function
   'mew-summary-pick-with-grep
@@ -105,7 +116,7 @@ This function MUST return text in which each line begins with a file name.")
       (goto-char (point-max))
       (setq nxt (nthcdr mew-prog-grep-max-msgs msgs))
       (if nxt (setcdr (nthcdr (1- mew-prog-grep-max-msgs) msgs) nil))
-      (apply (function call-process)
+      (apply 'call-process
 	     mew-prog-grep nil t nil
 	     (append mew-prog-grep-opts (list pattern) msgs))
       (setq msgs nxt))))
@@ -126,19 +137,19 @@ This function MUST return text in which each line begins with a file name.")
 	(cd dir)
 	(mew-piolet
 	 mew-cs-text-for-read mew-cs-text-for-write
-	 (let ((file-name-coding-system nil)) ;; for XEmacs
-	   (funcall mew-summary-pick-with-external-function pattern msgs)))
+	 (mew-alet
+	  (funcall mew-summary-pick-with-external-function pattern msgs)))
 	(goto-char (point-min))
 	(setq msgs nil)
 	(while (not (eobp))
 	  (if (looking-at mew-regex-message-files2)
-	      (setq msgs (cons (mew-match 0) msgs)))
+	      (setq msgs (cons (mew-match-string 0) msgs)))
 	  (forward-line)))
       (mew-remove-buffer buf)
       (setq msgs (mew-uniq-list msgs))
-      (setq msgs (mapcar (function string-to-int) msgs))
-      (setq msgs (sort msgs (function <)))
-      (mapcar (function int-to-string) msgs))))
+      (setq msgs (mapcar 'string-to-int msgs))
+      (setq msgs (sort msgs '<))
+      (mapcar 'int-to-string msgs))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -260,11 +271,11 @@ keyword, execute this command with '\\[universal-argument]'."
   ;; #1 can be replaced by #.
   (let (args body asc value)
     (while (string-match "\\(#[0-9]*\\)[, ]*" str1)
-      (setq args (cons (intern (mew-match 1 str1)) args))
+      (setq args (cons (intern (match-string 1 str1)) args))
       (setq str1 (replace-match "" nil t str1)))
     (while (string-match "#[0-9]*" str2)
       (setq body (cons (substring str2 0 (match-beginning 0)) body))
-      (setq body (cons (intern (mew-match 0 str2)) body))
+      (setq body (cons (intern (match-string 0 str2)) body))
       (setq str2 (substring str2 (match-end 0))))
     (setq body (cons str2 body))
     (setq asc (assoc str1 mew-pick-macro-alist))
@@ -293,7 +304,7 @@ keyword, execute this command with '\\[universal-argument]'."
 	(setq body (cdr body)))
       (concat "("
 	      (mew-pick-macro-expand-string
-	       (apply (function concat) (nreverse body-copy)))
+	       (apply 'concat (nreverse body-copy)))
 	      ")"))))
 
 (defun mew-pick-macro-expand-string (str)
@@ -305,7 +316,7 @@ keyword, execute this command with '\\[universal-argument]'."
 	  (concat (char-to-string first)
 		  (mew-pick-macro-expand-string (substring str 1)))
 	(if (string-match "=\\| \\|)\\|&\\||" str)
-	    (if (string= (mew-match 0 str) "=")
+	    (if (string= (match-string 0 str) "=")
 		(progn
 		  (setq eq-flag t)
 		  (setq key (substring str 0 (match-end 0)))
