@@ -7,6 +7,39 @@
 
 (require 'mew)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Scan info
+;;;
+
+(defvar mew-scan-info-list '("folder" "message"))
+;; See mew-scan-fields. 0th is fld, 1st is msg (ie num).
+
+(mew-info-defun "mew-scan-" mew-scan-info-list)
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Variables
+;;;
+
+(defvar mew-scan-form-size-unit '("" "k" "M" "G" "T"))
+(defvar mew-vec [0 1 2 3 4 5 6 8 9 10 11 12 13 14 15 16 17 18 19 20]
+  "Just for test of (MEW-FOO).")
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Setup
+;;;
+
+(defun mew-scan-setup ()
+  "Define functions (MEW-FOO) according 'mew-scan-fields-alias'."
+  (let ((n (length mew-scan-fields-alias))
+	(i 0))
+    (while (< i n)
+      (fset (intern (concat "MEW-" (nth i mew-scan-fields-alias)))
+	    `(lambda () (aref mew-vec ,i)))
+      (setq i (1+ i)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Pre-defined functions for mew-scan-form
@@ -27,7 +60,7 @@ effect to this function."
   (let ((id (MEW-ID)) duplicated review)
     (when mew-scan-form-mark-delete
       (when (string-match mew-regex-id id)
-	(setq id (mew-match 1 id))
+	(setq id (match-string 1 id))
 	(if (member id (mew-sinfo-get-scan-id)) ;; in Summary mode
 	    (setq duplicated t)
 	  (mew-sinfo-set-scan-id (cons id (mew-sinfo-get-scan-id))))))
@@ -47,10 +80,9 @@ effect to this function."
 (defun mew-scan-form-type ()
   "A function to return a mark of content type."
   (let ((ct (MEW-CT))
-	(uid (mew-scan-usc-uid (mew-scan-split-usc (MEW-UIDL))))
 	(case-fold-search t))
     (cond
-     ((not (string= uid ""))                       "T")
+     ((mew-scan-message-truncatedp)                "T")
      ((string-match "Multipart/Signed"    ct)      "S")
      ((string-match "Multipart/Encrypted" ct)      "E")
      ((string-match "Application/X-Pkcs7-Mime" ct) "E")
@@ -63,10 +95,13 @@ effect to this function."
   (let ((s (MEW-DATE)))
     (if (or (string= s "")
 	    (not (string-match mew-time-rfc-regex s)))
-	"00:00"
-      (format "%02d:%2s"
-	      (or (mew-time-rfc-hour) 0)
-	      (or (mew-time-rfc-min) "00")))))
+	(setq s (mew-time-ctz-to-rfc
+		 (mew-file-get-time (mew-expand-folder (MEW-FLD) (MEW-NUM))))))
+    (if (string-match mew-time-rfc-regex s)
+	(format "%02d:%02d"
+		(or (mew-time-rfc-hour) 0)
+		(or (mew-time-rfc-min)  0))
+      "00:00")))
 
 (defun mew-scan-form-date ()
   "A function to return a date, MM/DD."
@@ -95,15 +130,13 @@ effect to this function."
 	(setq year (+ year 1900))))
       (int-to-string year))))
 
-(defvar mew-scan-form-size-unit '("" "k" "M" "G" "T"))
-
 (defun mew-scan-form-size ()
   "A function to return the size of the message. Should be used
 with -4. See also 'mew-scan-form-size-0k' and 'mew-scan-form-size-huge'."
   (let ((len-1 (1- (length mew-scan-form-size-unit)))
-	(SIZE (mew-scan-usc-size (mew-scan-split-usc (MEW-UIDL))))
+	(SIZE (mew-scan-uid-size (MEW-UID)))
 	(i 0) size unit)
-    (if (string-match "^[0-9]+$" SIZE)
+    (if (and SIZE (string-match "^[0-9]+$" SIZE))
 	(setq size (string-to-int SIZE))
       (setq size (mew-file-get-size (mew-expand-folder (MEW-FLD) (MEW-NUM)))))
     (while (and (< i len-1) (>= size 1000))
@@ -136,17 +169,17 @@ with -4. See also 'mew-scan-form-size-0k' and 'mew-scan-form-size-huge'."
 	 ((and (eq rule 'name)
 	       (or (string-match "^\"\\([^\"]+\\)\"[ \t]*<[^>]+>" addr)
 		   (string-match "^\\([^<]+\\)<[^>]+>" addr)))
-	  (throw 'matched (setq ret (mew-chop (mew-match 1 addr)))))
+	  (throw 'matched (setq ret (mew-chop (match-string 1 addr)))))
 	 ((and (eq rule 'comment)
 	       (string-match "^[^(]+(\\(.+\\))" addr))
-	  (throw 'matched (setq ret (mew-chop (mew-match 1 addr)))))
+	  (throw 'matched (setq ret (mew-chop (match-string 1 addr)))))
 	 ((eq rule 'address)
 	  (throw 'matched (setq ret raw)))
 	 ((and (eq rule 'nickname) nickname)
 	  (throw 'matched (setq ret nickname)))
 	 ((and (stringp rule)
 	       (string-match rule addr))
-	  (throw 'matched (setq ret (mew-chop (mew-match 1 addr))))))))
+	  (throw 'matched (setq ret (mew-chop (match-string 1 addr))))))))
     (or ret addr)))
 
 (defun mew-scan-form-from ()
@@ -185,130 +218,16 @@ Address is converted by 'mew-scan-form-extract-addr'. See also
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; The engine function to call mew-scan-form-*
-;;;
-
-(defsubst mew-scan-split-usc (xuidl)
-  (mew-split xuidl 32))
-
-(defsubst mew-scan-usc-uid (usc)
-  (or (nth 0 usc) ""))
-
-(defsubst mew-scan-usc-size (usc)
-  (or (nth 1 usc) ""))
-
-(defsubst mew-scan-usc-case (usc)
-  (or (nth 2 usc) ""))
-
-(defun mew-scan-get-piece (spec)
-  (let (func width str rightp)
-    (if (stringp spec)
-	(progn
-	  (setq TOTAL (+ TOTAL (length spec))) ;; xxx length
-	  spec)
-      (if (symbolp spec)
-	  (setq width 1 func spec)
-	(setq width (nth 0 spec) func (nth 1 spec)))
-      (when (and (integerp width) (symbolp func))
-	(if (= width 0) (setq width (- WIDTH TOTAL 1)))
-	(if (< width 0) (setq width (abs width) rightp t))
-	(setq TOTAL (+ TOTAL width))
-	(setq func (intern-soft
-		    (concat mew-scan-form-func-prefix (symbol-name func))))
-	(when (fboundp func)
-	  (setq str (funcall func))
-	  (if rightp
-	      (if (<= (length str) width)
-		  (format (format "%%%ds" width) str)
-		(setq TOTAL (+ (- TOTAL width) (length str)))
-		str) ;; width may exceed.
-	    (mew-substring str width)))))))
-
-(defun mew-scan-get-line (mew-vec WIDTH &optional VIRTUALP)
-  (let* ((TOTAL 0)
-	 (line (mapconcat (function mew-scan-get-piece)
-			  (mew-sinfo-get-scan-form) ""))
-	 (my-id "") (par-id "") (first-irt "") fld msg ld onlyone start)
-    (if (string-match mew-regex-id (MEW-ID))
-	(setq my-id (mew-match 1 (MEW-ID))))
-    ;; RFC 2822 says: the "In-Reply-To:" field may be used to identify
-    ;; the message (or messages) to which the new message is a reply,
-    ;; while the "References:" field may be used to identify a
-    ;; "thread" of conversation.
-    ;;
-    ;; However, even if the References field exists, it may not contain
-    ;; a parent's ID. So, if the In-Reply-To field contain one ID, 
-    ;; we use it for thread.
-    ;;
-    ;; (1) The In-Reply-To contains one ID, use it.
-    ;; (2) The References contains one or more IDs, use the last one.
-    ;; (3) The In-Reply-To contains two or more IDs, use the first one.
-    (when (string-match mew-regex-id (MEW-IRT))
-      (setq start (match-end 0))
-      (setq first-irt (mew-match 1 (MEW-IRT)))
-      (unless (string-match mew-regex-id (MEW-IRT) start)
-	(setq onlyone t)))
-    (cond
-     (onlyone
-      (setq par-id first-irt))
-     ((string-match mew-regex-id (MEW-REF))
-      (setq start (match-end 0))
-      (setq par-id (mew-match 1 (MEW-REF)))
-      (while (string-match mew-regex-id (MEW-REF) start)
-	(setq start (match-end 0))
-	(setq par-id (mew-match 1 (MEW-REF)))))
-     (t
-      (setq par-id first-irt)))
-    (if (null VIRTUALP)
-	(setq ld (format "\r <%s> <%s>\n" my-id par-id))
-      (setq fld (mew-scan-get-folder mew-vec))
-      (setq msg (mew-scan-get-message mew-vec))
-      (setq ld (format "\r %s %s <%s> <%s>\n" fld msg my-id par-id)))
-    (cons line ld)))
-
-;;
-
-(defvar mew-vec [0 1 2 3 4 5 6 8 9 10 11 12 13 14 15 16 17 18 19 20]
-  "Just for test of (MEW-FOO).")
-
-(defun mew-scan-setup ()
-  "Define functions (MEW-FOO) according 'mew-scan-fields-alias'."
-  (let ((n (length mew-scan-fields-alias))
-	(i 0))
-    (while (< i n)
-      (fset (intern (concat "MEW-" (nth i mew-scan-fields-alias)))
-	    (list 'lambda () (list 'aref 'mew-vec i)))
-      (setq i (1+ i)))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;;
+;;; Scan from
 ;;;
 
 (defun mew-summary-scan-form (folder &optional column)
   "Get scan-form from 'mew-summary-refine-scan-form'.
 See also 'mew-scan-form-list-list'."
-  (let ((lst mew-scan-form-list)
-	(str-func (mew-scan-form-list-string-func))
-	(lst-func (mew-scan-form-list-list-func))
-	key keys values ret form col)
-    (catch 'loop
-      (while lst
-	(setq keys (car (car lst)))
-	(setq values (cdr (car lst)))
-	(setq lst (cdr lst))
-	(cond
-	 ((eq keys t)
-	  (throw 'loop (setq ret values)))
-	 ((stringp keys)
-	  (if (funcall str-func keys folder)
-	      (throw 'loop (setq ret values))))
-	 ((listp keys)
-	  (while keys
-	    (setq key (car keys))
-	    (setq keys (cdr keys))
-	    (if (funcall lst-func key folder)
-		(throw 'loop (setq ret values))))))))
+  (let (ret form col)
+    (setq ret (mew-folder-spec folder mew-scan-form-list
+			       mew-scan-form-list-string-type
+			       mew-scan-form-list-list-type))
     (if ret
 	(setq form (nth 0 ret) col (nth 1 ret))
       (setq form mew-scan-form))
@@ -348,257 +267,138 @@ Otherwise append 'mew-scan-form-header' to FORM."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Exchanging the cursor
+;;; The engine function to call mew-scan-form-*
 ;;;
 
-(defvar mew-summary-inbox-position (make-marker))
+(defun mew-scan-get-piece (spec)
+  (let (func width str rightp)
+    (if (stringp spec)
+	(progn
+	  (setq TOTAL (+ TOTAL (length spec))) ;; xxx length
+	  spec)
+      (if (symbolp spec)
+	  (setq width 1 func spec)
+	(setq width (nth 0 spec) func (nth 1 spec)))
+      (when (and (integerp width) (symbolp func))
+	(if (= width 0) (setq width (- WIDTH TOTAL 1)))
+	(if (< width 0) (setq width (abs width) rightp t))
+	(setq TOTAL (+ TOTAL width))
+	(setq func (intern-soft
+		    (concat mew-scan-form-func-prefix (symbol-name func))))
+	(when (fboundp func)
+	  (setq str (funcall func))
+	  (if rightp
+	      (if (<= (length str) width)
+		  (format (format "%%%ds" width) str)
+		(setq TOTAL (+ (- TOTAL width) (length str)))
+		str) ;; width may exceed.
+	    (mew-substring str width)))))))
 
-(defun mew-summary-exchange-point ()
-  "Get back to the position before typing '\\<mew-summary-mode-map>\\[mew-summary-retrieve]'."
-  (interactive)
-  (mew-summary-only
-   (cond
-    ((not (mew-folder-inboxp (mew-summary-folder-name 'ext)))
-     (message "Cannot be used in this folder"))
-    ((not (marker-position mew-summary-inbox-position))
-     (message "No previous folder"))
-    (t
-     (let* ((buf (marker-buffer mew-summary-inbox-position))
-	    (pos (marker-position mew-summary-inbox-position))
-	    (folder (buffer-name buf))
-	    getback)
-       (cond
-	((equal (current-buffer) buf)
-	 (goto-char pos))
-	((not (get-buffer buf))
-	 (message "Previous folder not exist"))
-	((mew-folder-virtualp folder)
-	 (if (mew-virtual-thread-p folder)
-	     (if (mew-thread-cache-valid-p folder)
-		 (setq getback t)
-	       (message "%s is old. " folder))
-	   (setq getback t)))
-	(t
-	 (setq getback t)))
-       (when getback
-	 (switch-to-buffer buf)
-	 (goto-char pos)))))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Getting messages
-;;;
-
-(defun mew-range-erase-p (range)
-  (or (char-equal (aref range 0) (aref mew-range-all 0))
-      (string-match ":" range)))
-
-(defun mew-summary-retrieve (&optional no-flush)
-  "Retrieve messages to +inbox asynchronously.
-If called with '\\[universal-argument]', +queue is not flushed."
-  (interactive "P")
-  (let* ((case mew-case-input)
-	 (inbox (mew-inbox-folder case))
-	 (mailbox (mew-mailbox-type case))
-	 range)
-    ;; for mew-summary-exchange-point.
+(defun mew-scan-get-line (mew-vec WIDTH &optional VIRTUALP)
+  (let* ((TOTAL 0)
+	 (line (mapconcat 'mew-scan-get-piece (mew-sinfo-get-scan-form) ""))
+	 (my-id "") (par-id "") (first-irt "")
+	 fld msg ld onlyone start uid siz)
+    (if (string-match mew-regex-id (MEW-ID))
+	(setq my-id (match-string 1 (MEW-ID))))
+    ;; RFC 2822 says: the "In-Reply-To:" field may be used to identify
+    ;; the message (or messages) to which the new message is a reply,
+    ;; while the "References:" field may be used to identify a
+    ;; "thread" of conversation.
+    ;;
+    ;; However, even if the References field exists, it may not contain
+    ;; a parent's ID. So, if the In-Reply-To field contain one ID, 
+    ;; we use it for thread.
+    ;;
+    ;; (1) The In-Reply-To contains one ID, use it.
+    ;; (2) The References contains one or more IDs, use the last one.
+    ;; (3) The In-Reply-To contains two or more IDs, use the first one.
+    (when (string-match mew-regex-id (MEW-IRT))
+      (setq start (match-end 0))
+      (setq first-irt (match-string 1 (MEW-IRT)))
+      (unless (string-match mew-regex-id (MEW-IRT) start)
+	(setq onlyone t)))
     (cond
-     ((mew-summary-or-virtual-p)
-      (set-marker mew-summary-inbox-position (point) (current-buffer))
-      (mew-summary-switch-to-folder inbox))
+     (onlyone
+      (setq par-id first-irt))
+     ((string-match mew-regex-id (MEW-REF))
+      (setq start (match-end 0))
+      (setq par-id (match-string 1 (MEW-REF)))
+      (while (string-match mew-regex-id (MEW-REF) start)
+	(setq start (match-end 0))
+	(setq par-id (match-string 1 (MEW-REF)))))
      (t
-      (mew-summary-switch-to-folder inbox)
-      (set-marker mew-summary-inbox-position (point) (current-buffer))))
-    ;; in the inbox
-    (when (mew-summary-exclusive-p)
-      (when (mew-summary-folder-dir-newp)
-	;; scan the gap
-	(goto-char (point-max))
-	(setq range (mew-input-range inbox mew-ask-range))
-	(if (mew-range-erase-p range)
-	    (mew-erase-buffer))
-	(mew-scan (mew-scan-mewls-src inbox range))
-	(mew-rendezvous mew-summary-buffer-process))
-      (mew-window-configure 'summary)
-      (mew-current-set nil nil nil)
-      (mew-decode-syntax-delete)
-      (goto-char (point-max))
-      (mew-sinfo-set-start-point (point))
-      (mew-sinfo-set-direction 'down)
-      (mew-sinfo-set-scan-form (mew-summary-scan-form inbox))
-      (cond
-       ((eq mailbox 'pop)
-	(mew-pop-retrieve case 'inc (not no-flush)))
-       ((eq mailbox 'mbox)
-	(mew-mbox-retrieve case))))))
+      (setq par-id first-irt)))
+    (setq uid (mew-scan-uid-uid (MEW-UID)))
+    (cond
+     (VIRTUALP
+      (setq fld (mew-scan-get-folder mew-vec))
+      (setq fld (cdr (assoc fld (mew-vinfo-get-lra))))
+      (setq msg (mew-scan-get-message mew-vec))
+      (setq ld (format "\r <%s> <%s> %s %s \006 %s %s\n"
+		       my-id par-id "" "" fld msg)))
+     (uid
+      (setq siz (mew-scan-uid-size (MEW-UID)))
+      (setq ld (format "\r <%s> <%s> %s %s\n" my-id par-id uid siz)))
+     (t
+      (setq ld (format "\r <%s> <%s>\n" my-id par-id))))
+    (cons line ld)))
 
-(defun mew-header-get-xuidl-from-file (fld msg)
-  (let ((buf (generate-new-buffer mew-buffer-prefix))
-	(xmu mew-x-mew-uidl:)
-	xuidl)
+;; See also mew-summary-cook-region
+(defun mew-scan-insert-line (folder vec width lmsg virtualp)
+  (when (get-buffer folder)
     (save-excursion
-      (set-buffer buf)
-      (mew-erase-buffer)
-      (mew-insert-message
-       fld msg mew-cs-text-for-read mew-header-reasonable-size)
-      (setq xuidl (mew-header-get-value xmu)))
-    (mew-remove-buffer buf)
-    xuidl))
-
-(defun mew-summary-retrieve-message (&optional rev-del)
-  "Retrieve the rest of a truncated('T') message.
-If 'mew-pop-delete' is 't', delete the message from the mailbox.
-When executed with '\\[universal-argument]', 'mew-pop-delete' is considered reversed."
-  (interactive "P")
-  (mew-summary-msg
-   (mew-summary-only
-    (mew-summary-not-in-mdrop
-     (when (mew-summary-exclusive-p)
-       (let* ((fld (mew-summary-folder-name 'ext))
-	      (msg (mew-summary-message-number))
-	      (xuidl (mew-header-get-xuidl-from-file fld msg))
-	      usc uid siz case del rtr rtrs)
-	 (if (null xuidl)
-	     (message "No %s field" mew-x-mew-uidl:)
-	   (setq usc (mew-scan-split-usc xuidl))
-	   (setq uid (mew-scan-usc-uid usc))
-	   (setq siz (mew-scan-usc-size usc))
-	   (setq case (mew-scan-usc-case usc))
-	   (setq del (eq (mew-pop-delete case) t))
-	   (if rev-del (setq del (not del)))
-	   (setq rtr (list uid siz del (list fld msg)))
-	   (setq rtrs (list rtr))
-	   (mew-sinfo-set-scan-form (mew-summary-scan-form fld))
-	   (mew-pop-retrieve case 'get fld rtrs))))))))
-
-(defun mew-summary-mark-retrieve-message (&optional rev-del)
-  "Retrieve the rest of truncated('T') messages marked with '*'.
-If 'mew-pop-delete' is 't', delete the message from the mailbox.
-When executed with '\\[universal-argument]', 'mew-pop-delete' is considered reversed."
-  (interactive "P")
-  (mew-summary-only
-   (mew-summary-not-in-mdrop
-    (let* ((fld (mew-summary-folder-name 'ext))
-	   (msgs (mew-summary-mark-collect mew-mark-review))
-	   msg usc xuidl uid siz case del rtr rtrs)
-      (if (null msgs)
-	  (message "No marked messages")
-	(if rev-del (setq del (not del)))
-	(while msgs
-	  (setq msg (car msgs))
-	  (setq msgs (cdr msgs))
-	  (setq xuidl (mew-header-get-xuidl-from-file fld msg))
-	  (when xuidl
-	    (setq usc (mew-scan-split-usc xuidl))
-	    (setq uid (mew-scan-usc-uid usc))
-	    (setq siz (mew-scan-usc-size usc))
-	    (setq case (mew-scan-usc-case usc)) ;; xxx
-	    (setq del (eq (mew-pop-delete case) t))
-	    (if rev-del (setq del (not del)))
-	    (setq rtr (list uid siz del (list fld msg)))
-	    (setq rtrs (cons rtr rtrs)))))
-      (if (null rtrs)
-	  (message "No %s field" mew-x-mew-uidl:)
-	(mew-sinfo-set-scan-form (mew-summary-scan-form fld))
-	(mew-pop-retrieve case 'get fld rtrs))))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Retrieving messages from mbox
-;;;
-
-(defun mew-mbox-retrieve (case)
-  (let ((mbox-command (mew-mbox-command case))
-	(mbox-command-arg (mew-mbox-command-arg case))
-	(inbox (mew-inbox-folder case))
-	args)
-    (if (not (mew-which-exec mbox-command))
-	(message "'%s' not found!" mbox-command)
-      (if mbox-command-arg
-	  (setq args (list "-e" mbox-command "-m" mbox-command-arg))
-	(setq args (list "-e" mbox-command)))
-      (setq args (append args (mew-scan-mewls-src inbox)))
-      (mew-scan args  nil nil 'flush))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Scanning a folder
-;;;
-
-(defun mew-summary-ls (&optional jump)
-  "List this folder asynchronously.
-If executed with '\\[universal-argument]', the cursor stays in the
-current position."
-  (interactive "P")
-  (when (mew-summary-exclusive-p)
-    (mew-summary-only
-     (let* ((folder (mew-summary-folder-name 'ext))
-	    (askp mew-ask-range)
-	    scanp range mkdb oldformatp)
-       (mew-summary-folder-cache-load)
-       (save-excursion
-	 (goto-char (point-min))
-	 (if (and (not (eobp))
-		  (not (looking-at mew-regex-thread))
-		  (y-or-n-p "The format of this scan cache is old. Type 'y' when ready. "))
-	     (setq oldformatp t)))
-       (if oldformatp
-	   (progn
-	     (setq range mew-range-all)
-	     (setq mkdb (mew-summary-mark-collect4))
-	     (mew-erase-buffer)
-	     (mew-scan (mew-scan-mewls-src folder range) mkdb))
-	 (cond
-	  ((interactive-p);; "s"
-	   (setq askp t)
-	   (setq scanp t))
-	  ((mew-summary-folder-dir-newp);; "g"
-	   (setq askp nil)
-	   (setq scanp t)))
-	 (set-buffer-modified-p nil)
-	 (if (or (interactive-p) jump) (goto-char (point-max)))
-	 (if (not scanp)
-	     (run-hooks 'mew-summary-ls-no-scan-hook)
-	   ;; asking range when "s"
-	   (setq range (mew-input-range folder askp))
-	   (when (mew-range-erase-p range)
-	     (setq mkdb (mew-summary-mark-collect4))
-	     (mew-erase-buffer))
-	   (if (mew-folder-mdropp folder)
-	       (mew-scan-mdrop folder)
-	     (mew-scan (mew-scan-mewls-src folder range) mkdb))))))))
-
-(defun mew-scan-mdrop (mdrop)
-  (let* ((mdropdir (mew-expand-folder mdrop))
-	 (msgs (mew-dir-messages mdropdir)))
-    (mew-summary-clean-msgs mdrop msgs))
-  (mew-sinfo-set-scan-form (mew-summary-scan-form mdrop))
-  (mew-pop-retrieve mew-case-input 'scan mdrop))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Scan info
-;;;
-
-(defvar mew-scan-info-list '("folder" "message"))
-
-(mew-info-defun "mew-scan-" mew-scan-info-list)
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Process name
-;;;
-
-(defconst mew-scan-info-prefix "mew-scan-info-")
-
-(defsubst mew-scan-info-name (bnm)
-  (format "%s<%s>" mew-scan-info-prefix bnm))
-
-(defsubst mew-scan-pnm-to-bnm (pnm)
-  (if (string-match mew-regex-id pnm) (mew-match 1 pnm)))
-
-(defsubst mew-scan-buffer-name (folder)
-  (concat mew-buffer-prefix folder))
+      (set-buffer folder)
+      (let* ((line (mew-scan-get-line vec width virtualp))
+	     (opos (point))
+	     (omax (point-max))
+	     beg med face mark olen nlen)
+	(mew-elet
+	 (if (null lmsg)
+	     (goto-char (point-max))
+	   ;; a message marked with 'T'.
+	   (goto-char (point-min))
+	   (if (not (re-search-forward (mew-regex-jmp-msg lmsg) nil t))
+	       (goto-char (point-max)) ;; xxx
+	     (beginning-of-line)
+	     (setq beg (point))
+	     (forward-line)
+	     ;; To avoid inserting a line AFTER the cursor underline,
+	     ;; keep this line and make it invisible.
+	     (put-text-property beg (point) 'invisible t)
+	     (forward-line -1)))
+	 (setq beg (point))
+	 ;; To "insert" just after mew-marker-decode-syntax-end.
+	 (insert (car line))
+	 (setq med (point))
+	 (goto-char beg)
+	 (mew-front-nonsticky beg med) ;; for XEmacs
+	 (if (and mew-use-highlight-mark
+		  (looking-at mew-regex-msg-mark)
+		  (setq mark (string-to-char (mew-match-string 2)))
+		  (setq face (mew-highlight-mark-get-face mark)))
+	     (put-text-property beg med 'face face))
+	 (if mew-use-highlight-mouse-line
+	     (put-text-property
+	      beg med 'mouse-face mew-highlight-mouse-line-face))
+	 (goto-char med)
+	 (insert (cdr line))
+	 (put-text-property med (1- (point)) 'invisible t)
+	 ;; Removing the invisible line.
+	 (when lmsg
+	   ;; UID information will be removed. So, we need to adjust
+	   ;; the position.
+	   (setq nlen (- (point) beg))
+	   (setq beg (point))
+	   (forward-line)
+	   (when (> opos beg)
+	     (setq olen (- (point) beg))
+	     (setq opos (- opos (- olen nlen))))
+	   (delete-region beg (point))))
+	(if (or (eq opos (mew-sinfo-get-start-point))
+		(/= opos omax))
+	    ;; move the cursor to the original position.
+	    (goto-char opos))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -614,15 +414,15 @@ current position."
       (while (not (eobp))
 	(if (not (looking-at mew-keyval))
 	    (forward-line)
-	  (setq key (mew-capitalize (mew-match 1)))
+	  (setq key (mew-capitalize (mew-match-string 1)))
 	  (setq med (match-end 0))
 	  (forward-line)
 	  (mew-header-goto-next)
 	  (when (setq n (mew-member-case-equal key mew-scan-fields))
 	    ;; xxx this may be able to tune up.
 	    (mew-header-decode-region key med (point) draftp)
-	    ;; Need to inherit properties.
-	    ;; So, we use buffer-substring, not mew-buffer-substring.
+	    ;; We need to keep composite properties of charset.
+	    ;; This must be "buffer-substring".
 	    (setq str (buffer-substring med (1- (point))))
 	    (aset vec n str))))
       (goto-char (point-max)))
@@ -638,220 +438,153 @@ current position."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Scan body
+;;; X-Mew-Uidl:
 ;;;
 
-(defun mew-scan-mewls-src (folder &optional range)
-  (if range
-      (list "-s" (format "%s %s" folder range))
-    (list "-s" folder)))
+(defsubst mew-scan-uid-uid (uid)
+  (nth 0 (mew-split uid 32)))
 
-(defun mew-scan (args &optional mkdb virtualp flush)
-  (cond
-   ((not (mew-summary-exclusive-p))
-    ())
-   ((not (mew-which-exec mew-prog-mewls))
-    (message "'%s' not found!" mew-prog-mewls))
-   (t
-    (let ((process-connection-type mew-connection-type2)
-	  process buf bnm)
-      (condition-case nil
-	  (progn
-	    (cond
-	     (virtualp
-	      (or (mew-virtual-p) (mew-virtual-mode)))
-	     (t 
-	      (or (mew-summary-p) (mew-summary-mode))))
-	    (mew-window-configure 'summary)
-	    (mew-current-set nil nil nil)
-	    (mew-decode-syntax-delete)
-	    (setq bnm (mew-summary-folder-name 'ext))
-	    (mew-sinfo-set-scan-form (mew-summary-scan-form bnm))
-	    (mew-sinfo-set-start-point (point))
-	    (mew-sinfo-set-direction 'down)
-	    (mew-sinfo-set-scan-id nil)
-	    (mew-sinfo-set-mkdb mkdb)
-	    (mew-sinfo-set-flush flush)
-	    (message "Scanning %s..." bnm)
-	    (setq buf (get-buffer-create (mew-scan-buffer-name bnm)))
-	    (save-excursion
-	      (set-buffer buf)
-	      (mew-erase-buffer))
-	    (setq args (append (list "-b" mew-mail-path "-c" mew-news-path
-				     "-w" (int-to-string mew-scan-wait-for)
-				     "-f" (mapconcat (function identity)
-						     (nthcdr 2 mew-scan-fields)
-						     ","))
-			       args))
-	    (setq process (apply (function start-process)
-				 (mew-scan-info-name bnm)
-				 buf mew-prog-mewls args))
-	    (mew-summary-lock process "Scanning")
-	    (mew-set-process-cs process mew-cs-text-for-read mew-cs-dummy)
-	    ;; text may be broken, so undecided is very dangerous!
-	    (set-process-filter   process 'mew-scan-filter)
-	    (set-process-sentinel process 'mew-scan-sentinel)
-	    (process-kill-without-query process))
-	(quit
-	 (set-process-sentinel process nil)
-	 (mew-sinfo-set-start-point nil)
-	 (mew-sinfo-set-mkdb nil)
-	 (mew-sinfo-set-flush nil)
-	 (mew-summary-unlock)))))))
+(defsubst mew-scan-uid-size (uid)
+  (nth 1 (mew-split uid 32)))
 
-;; See also mew-summary-cook-region
-(defun mew-scan-insert-line (folder vec width lmsg virtualp)
-  (when (get-buffer folder)
-    (save-excursion
-      (set-buffer folder)
-      (let* ((after-change-functions nil)
-	     (line (mew-scan-get-line vec width virtualp))
-	     (opos (point))
-	     (omax (point-max))
-	     beg med face mark)
-	(mew-elet
-	 (if (null lmsg)
-	     (goto-char (point-max))
-	   ;; a message marked with 'T'.
-	   (goto-char (point-min))
-	   (if (not (re-search-forward (mew-regex-jmp-msg lmsg) nil t))
-	       (goto-char (point-max));; xxx
-	     (beginning-of-line)
-	     (setq beg (point))
-	     (forward-line)
-	     ;; To avoid inserting a line AFTER the cursor underline,
-	     ;; keep this line and make it invisible.
-	     (put-text-property beg (point) 'invisible t)
-	     (forward-line -1)))
-	 (setq beg (point))
-	 ;; To "insert" just after mew-marker-decode-syntax-end.
-	 (insert (car line))
-	 (setq med (point))
-	 (goto-char beg)
-	 (mew-front-nonsticky beg med);; for XEmacs
-	 (if (and mew-use-highlight-mark
-		  (looking-at mew-regex-msg-mark)
-		  (setq mark (string-to-char (mew-match 2)))
-		  (setq face (mew-highlight-mark-get-face mark)))
-	     (put-text-property beg med 'face face))
-	 (if mew-use-highlight-mouse-line
-	     (put-text-property
-	      beg med 'mouse-face mew-highlight-mouse-line-face))
-	 (goto-char med)
-	 (insert (cdr line))
-	 (put-text-property med (1- (point)) 'invisible t)
-	 ;; Removing the invisible line.
-	 (when lmsg
-	   (setq beg (point))
-	   (forward-line)
-	   (delete-region beg (point))))
-	(if (or (eq opos (mew-sinfo-get-start-point))
-		(/= opos omax))
-	    ;; move the cursor to the original position.
-	    (goto-char opos))))))
+(defun mew-header-insert-xmu (uid siz truncated)
+  (when (and (stringp uid) (stringp siz))
+    (setq siz (int-to-string (string-to-int siz))) ;; removing 0
+    (let (fields)
+      (if truncated
+	  (setq fields (concat uid " 0" siz)) ;; e.g. 0500 == truncated
+	(setq fields (concat uid " " siz)))
+      (mew-header-insert mew-x-mew-uidl: fields 'no-fold))))
 
-(defun mew-scan-filter (process string)
-  (let* ((width (1- (mew-scan-width))) ;; xxx
-	 (pnm (process-name process))
-	 (folder (mew-scan-pnm-to-bnm pnm))
-	 (virtualp (mew-folder-virtualp folder))
-	 (draftp (mew-folder-draftp folder))
-	 vec)
-    (mew-filter
-     (goto-char (point-max))
-     (mew-set-buffer-multibyte nil)
-     (mew-elet (insert string))
-     (goto-char (point-min))
-     (while (and (re-search-forward mew-eoh nil t) (not (eobp)))
-       (mew-set-buffer-multibyte t)
-       (setq vec (mew-scan-header draftp))
-       (mew-set-buffer-multibyte nil)
-       (mew-scan-insert-line folder vec width nil virtualp)
-       (forward-line)
-       (delete-region (point-min) (point))))))
+(defun mew-scan-message-truncatedp ()
+  (let ((siz (mew-scan-uid-size (MEW-UID))))
+    (and siz (string-match "^0" siz))))
 
-(defun mew-scan-sentinel (process event)
-  (let* ((pnm (process-name process))
-	 (folder (mew-scan-pnm-to-bnm pnm))
-	 (virtualp (mew-folder-virtualp folder))
-	 opos mdb ent msg mrk flush)
-    (mew-filter
-     (mew-set-buffer-multibyte t)
-     (set-buffer folder)
-     (setq opos (point))
-     (setq mdb (mew-sinfo-get-mkdb))
-     (setq flush (mew-sinfo-get-flush))
-     (goto-char (point-min))
-     (while mdb
-       (setq ent (car mdb))
-       (setq mdb (cdr mdb))
-       (setq msg (car ent))
-       (setq mrk (cdr ent))
-       (when (re-search-forward (mew-regex-jmp-msg msg) nil t)
-	 (mew-summary-mark-as mrk 'force)
-	 (forward-line)))
-     (goto-char opos)
-     (if (and virtualp (mew-vinfo-get-func))
-	 (funcall (mew-vinfo-get-func)))
-     (mew-vinfo-set-func nil)
-     (mew-sinfo-set-start-point nil)
-     (mew-sinfo-set-mkdb nil)
-     (mew-sinfo-set-flush nil)
-     (mew-summary-folder-cache-save)
-     (set-buffer-modified-p nil)
-     (mew-summary-unlock)
-     (message "Scanning %s...done" folder)
-     (run-hooks 'mew-scan-sentinel-hook)
-     (when (and mew-auto-flush-queue flush)
-       (mew-smtp-flush-queue
-	(mew-queue-folder mew-case-output) mew-case-output)))))
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Scanning a folder
+;;;
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defun mew-summary-ls (&optional header-only goend)
+  "List this folder asynchronously.
+
+In a LOCAL folder: messages in the local folder are scanned according
+to the range which you specify.
+
+In a REMOTE folder: messages in the server's folder are cached
+according to the range which you specify. If
+'mew-pop-header-only'/'mew-imap-header-only'/'mew-nntp-header-only' is
+non-nil, only headers of messages are cacched. If executed with
+'\\[universal-argument]', these variables are considered reversed."
+  (interactive "P")
+  (when (mew-summary-exclusive-p)
+    (mew-summary-only
+     (let* ((bnm (mew-summary-folder-name 'ext))
+	    (case (mew-sinfo-get-case))
+	    (fld (mew-sinfo-get-folder))
+	    (askp mew-ask-range)
+	    (directive 'scan)
+	    (get-body (not header-only))
+	    scanp range oldformatp dir-newp)
+       (mew-summary-folder-cache-load)
+       (save-excursion
+	 (goto-char (point-min))
+	 (if (and (not (eobp))
+		  (not (looking-at mew-regex-summary))
+		  (mew-folder-localp fld) ;; just in case
+		  (y-or-n-p "The format of this scan cache is old. Type 'y' when ready. "))
+	     (setq oldformatp t)))
+       (if oldformatp
+	   (mew-local-retrieve 'scan bnm mew-range-all 'erase)
+	 (cond
+	  ((interactive-p) ;; "s"
+	   (setq askp t)
+	   (setq scanp t))
+	  ((mew-summary-folder-dir-newp) ;; "g"
+	   (setq askp nil)
+	   (setq scanp t)))
+	 (if (mew-summary-folder-dir-newp) (setq dir-newp t))
+	 (if (or (interactive-p) goend) (goto-char (point-max)))
+	 (set-buffer-modified-p nil)
+	 (if (not scanp)
+	     (run-hooks 'mew-summary-ls-no-scan-hook)
+	   (mew-window-configure 'summary)
+	   ;;
+	   (mew-current-set nil nil nil)
+	   (mew-decode-syntax-delete)
+	   (mew-unhighlight-cursor-line)
+	   ;;
+	   (mew-sinfo-set-direction 'down)
+	   (cond
+	    ((and (mew-folder-remotep fld)
+		  (not (mew-folder-imap-queuep)))
+	     (if (and dir-newp (mew-folder-imapp fld))
+		 (mew-local-retrieve 'scan bnm (mew-range-update bnm))
+	       (setq range (mew-input-range-remote bnm))
+	       (when (eq range 'sync)
+		 (setq range nil)
+		 (setq directive 'sync))
+	       (cond
+		((mew-folder-popp fld)
+		 (if (mew-pop-header-only case)
+		     (setq get-body (not get-body)))
+		 (mew-pop-retrieve  case directive bnm range get-body))
+		((mew-folder-imapp fld)
+		 (if (mew-imap-header-only case)
+		     (setq get-body (not get-body)))
+		 (mew-imap-retrieve case directive bnm range get-body))
+		((mew-folder-nntpp fld)
+		 (if (mew-nntp-header-only case)
+		     (setq get-body (not get-body)))
+		 (mew-nntp-retrieve case directive bnm range get-body)))))
+	    (t ;; local
+	     (setq range (mew-input-range bnm askp))
+	     (if range
+		 (mew-local-retrieve 'scan bnm (nth 0 range) (nth 1 range))
+	       (message "range is wrong"))))))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Summary file cache
 ;;;
 
-(defun mew-summary-compare-times (type)
-  (let* ((folder (mew-summary-folder-name 'ext))
-	 (dir (file-chase-links (mew-expand-folder folder)))
-	 (tdir (if mew-touch-folder-p
-		   (mew-file-get-time
-		    (expand-file-name mew-summary-touch-file
-				      (mew-expand-folder dir)))
-		 (mew-file-get-time dir)))
-	 (cache (expand-file-name mew-summary-cache-file dir))
-	 (tcache (mew-file-get-time cache))
-	 (tbuf (mew-sinfo-get-cache-time))
-	 t1 t2)
-    (cond
-     ((eq type 'dir-cache)
-      (setq t1 tdir)
-      (setq t2 tcache))
-     ((eq type 'cache-buf)
-      (setq t1 tcache)
-      (setq t2 tbuf)))
-    (cond
-     ((null t1) nil)
-     ((null t2) t) ;; do update
-     ((> (nth 0 t1) (nth 0 t2)) t)
-     ((= (nth 0 t1) (nth 0 t2))
-      (if (> (nth 1 t1) (nth 1 t2)) t nil)) ;; nil if equal
-     (t nil))))
+(defun mew-compare-times (t1 t2)
+  ;; Is t1 newer than t2?
+  (cond
+   ((null t1) nil)
+   ((null t2) t) ;; do update
+   ((> (nth 0 t1) (nth 0 t2)) t)
+   ((= (nth 0 t1) (nth 0 t2))
+    (if (> (nth 1 t1) (nth 1 t2)) t nil)) ;; nil if equal
+   (t nil)))
 
 (defsubst mew-summary-folder-dir-newp ()
-  (mew-summary-compare-times 'dir-cache))
+  (let* ((folder (mew-summary-folder-name 'ext))
+	 (dir (file-chase-links (mew-expand-folder folder)))
+	 (t1 (if mew-touch-folder-p
+		 (mew-file-get-time
+		  (expand-file-name mew-summary-touch-file
+				    (mew-expand-folder dir)))
+	       (mew-file-get-time dir)))
+	 (cache (expand-file-name mew-summary-cache-file dir))
+	 (t2 (mew-file-get-time cache)))
+    (if (and mew-touch-folder-p (null t1)
+	     (file-directory-p dir)
+	     (mew-dir-messages (mew-expand-folder dir)))
+	t
+      (mew-compare-times t1 t2))))
 
 (defsubst mew-summary-folder-cache-newp ()
-  (mew-summary-compare-times 'cache-buf))
-
-(defsubst mew-summary-folder-cache-updatep (folder)
-  (mew-folder-localp folder))
+  (let* ((folder (mew-summary-folder-name 'ext))
+	 (cache (mew-expand-folder folder mew-summary-cache-file))
+	 (t1 (mew-file-get-time cache))
+	 (t2 (mew-sinfo-get-cache-time)))
+    (mew-compare-times t1 t2)))
 
 (defun mew-summary-folder-cache-load ()
   (let* ((folder (mew-summary-folder-name 'ext))
 	 (cache (mew-expand-folder folder mew-summary-cache-file)))
-    (when (and (mew-summary-folder-cache-updatep folder)
-	       (file-readable-p cache)
+    (when (and (file-readable-p cache)
 	       (mew-summary-folder-cache-newp))
       (mew-elet
        (mew-erase-buffer)
@@ -866,8 +599,7 @@ current position."
 (defun mew-summary-folder-cache-save ()
   (let* ((folder (mew-summary-folder-name 'ext))
 	 (cache (mew-expand-folder folder mew-summary-cache-file)))
-    (when (and (mew-summary-folder-cache-updatep folder)
-	       (file-writable-p cache))
+    (when (file-writable-p cache)
       (mew-touch-folder folder)
       (save-restriction
 	(widen)
@@ -887,12 +619,28 @@ current position."
 	       (write-region (point-min) (point-max) cache nil 'no-msg))
 	      (set-buffer cbuf)
 	      (mew-remove-buffer tmpbuf))
+	  ;; (write-region 1 1 ...) does not update the file timestamp
+	  ;; but does the directory timestamp. So, we need to delete
+	  ;; the file to update the file timestamp.
+	  (if (= (point-min) (point-max)) (mew-delete-file cache))
 	  (mew-frwlet
 	   mew-cs-dummy mew-cs-m17n
 	   (write-region (point-min) (point-max) cache nil 'no-msg))
-	  (set-file-modes cache mew-file-mode))
+	  (mew-set-file-modes cache))
 	(mew-sinfo-set-cache-time (mew-file-get-time cache))))))
 
+;; See also mew-net-folder-clean.
+(defun mew-summary-folder-cache-clean (folder)
+  "Erase Summary mode then remove and touch the cache file."
+  (if (get-buffer folder)
+      (save-excursion
+	(set-buffer folder)
+	(mew-erase-buffer)
+	(set-buffer-modified-p nil)))
+  (let ((cfile (mew-expand-folder folder mew-summary-cache-file)))
+    (if (file-exists-p cfile)
+	(write-region "" nil cfile nil 'no-msg))))
+
 (provide 'mew-scan)
 
 ;;; Copyright Notice:
