@@ -154,7 +154,7 @@
     (if (and (not (file-exists-p pubkey-moved))
 	     (file-writable-p pubkey-moved))
 	(copy-file pubkey-file pubkey-moved))
-    (delete-file pubkey-file)
+    (mew-delete-file pubkey-file)
     email-addr))
 
 ;; The followin function is based on mew-decode-multipart-encrypted.
@@ -212,10 +212,8 @@
 	 (put-text-property (point-min) (point-max) 'mew-noncontents nil)
 	 ;; because of RICH functionality of RFC1847... Gee dirty!
 	 (mew-decode-crlf-magic)))
-      (and encrypted-file (file-exists-p encrypted-file)
-	   (delete-file encrypted-file))
-      (and decrypted-file (file-exists-p decrypted-file)
-	   (delete-file decrypted-file))
+      (mew-delete-file encrypted-file)
+      (mew-delete-file decrypted-file)
       ;; Analyze the decrypted part
       (if (not result) syntax
 	(goto-char (point-min))
