@@ -319,24 +319,24 @@
       (setq str (toolbar-button-help-string (event-toolbar-button event)))
       ;; last-pressed-toolbar-button cannot be used.
       (if (string-match mew-regex-part-icon str)
-	  (setq ret (mew-match 1 str))))
+	  (setq ret (mew-match-string 1 str))))
     (or ret
 	(if (or (mew-in-attach-p)
 		(mew-in-decode-syntax-p))
 	    (save-excursion
 	      (beginning-of-line)
 	      (if (looking-at mew-regex-part)
-		  (mew-match 1)))))))
+		  (mew-match-string 1)))))))
 
 (defsubst mew-syntax-number-to-nums (strnum)
   (if strnum
-      (mapcar (function string-to-int) (mew-split strnum ?.))
+      (mapcar 'string-to-int (mew-split strnum ?.))
     nil))
 
 (defsubst mew-syntax-nums ()
   (mew-syntax-number-to-nums (mew-syntax-number)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; get number
 ;;;
@@ -358,7 +358,7 @@
 	  (setq str (toolbar-button-help-string 
 		     last-pressed-toolbar-button))
 	  (if (string-match mew-regex-msg-icon str)
-	      (setq ret (mew-match 1 str)))))
+	      (setq ret (mew-match-string 1 str)))))
     (if ret
 	ret
       (unless (mew-in-decode-syntax-p)
@@ -367,11 +367,11 @@
 	  (cond 
 	   ((mew-summary-p)
 	    (if (looking-at mew-regex-msg)
-		(mew-match 1)
+		(mew-match-string 1)
 	      nil))
 	   ((mew-virtual-p)
 	    (if (looking-at mew-regex-virtual)
-		(mew-match 2)
+		(mew-match-string 2)
 	      nil))
 	   (t nil)))))))
 
@@ -380,6 +380,22 @@
     (mew-summary-goto-message)
     (mew-summary-message-number)))
 
+(defun mew-summary-message-uid ()
+  (if (looking-at mew-regex-summary2)
+      (mew-match-string 3)))
+
+(defun mew-summary-message-size ()
+  (if (looking-at mew-regex-summary2)
+      (mew-match-string 4)))
+
+(defun mew-summary-my-id ()
+  (if (looking-at mew-regex-summary)
+      (mew-match-string 1)))
+
+(defun mew-summary-parent-id ()
+  (if (looking-at mew-regex-summary)
+      (mew-match-string 2)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; mew-encode-syntax
@@ -399,7 +415,7 @@
       (setq ctdb (mew-ctdb-by-file file))
       (setq ct (mew-ctdb-ct ctdb))
       (unless ct
-	(setq ct (mew-content-type)) ;; xxx case?
+	(setq ct (mew-content-type (mew-tinfo-get-case)))
 	(setq ctdb (mew-ctdb-by-ct ct)))
       (setq ctl (list ct))))
     (or cte (setq cte (mew-ctdb-cte ctdb)))
@@ -493,8 +509,7 @@
        (mew-xinfo-set-multi-form nil)
        (mew-xinfo-set-icon-spec nil)
        (mew-syntax-multipart syntax nil nil 'mew-draft-button)
-       (mapcar (function insert-and-inherit)
-	       (nreverse (mew-xinfo-get-multi-form)))
+       (mapcar 'insert-and-inherit (nreverse (mew-xinfo-get-multi-form)))
        (when mew-xemacs-p
 	 (goto-char (mew-attach-begin))
 	 (insert "X"))
@@ -529,7 +544,7 @@
 	(mew-elet
 	 (let ((pos (point)))
 	   (mew-decode-syntax-begin-set)
-	   (mapcar (function mew-decode-syntax-insert) form)
+	   (mapcar 'mew-decode-syntax-insert form)
 	   (if (= pos (point))
 	       ;; Nothing was printed.
 	       (mew-decode-syntax-remove)
@@ -814,7 +829,7 @@
 (defun mew-decode-syntax-arrange-warning ()
   (when (mew-xinfo-get-warning)
     (mew-xinfo-set-warning
-     (mew-uniq-list (sort (mew-xinfo-get-warning) (function string<))))))
+     (mew-uniq-list (sort (mew-xinfo-get-warning) 'string<)))))
 
 (defun mew-decode-syntax-insert-warning ()
   (when (or (mew-xinfo-get-warning) (mew-xinfo-get-info))
@@ -822,10 +837,10 @@
 	  (pref (make-string (1+ (length mew-x-mew:)) ?\ )))
       (when (mew-xinfo-get-warning)
 	(insert mew-x-mew: " "
-		(mapconcat (function identity) (mew-xinfo-get-warning) pref)))
+		(mapconcat 'identity (mew-xinfo-get-warning) pref)))
       (when (mew-xinfo-get-info)
 	(insert mew-x-mew: " "
-		(mapconcat (function identity) (mew-xinfo-get-info) pref)))
+		(mapconcat 'identity (mew-xinfo-get-info) pref)))
       (mew-decode-header-property-region beg (point)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
